#ifndef __ENTITY__
#define __ENTITY__

#include "mystuff/m3a.h"
#include "mystuff/GLVECTOR3f.h"


#define PRIORITY_NORMAL 100
#define PRIORITY_LATE 500
#define PRIORITY_EARLY 10


#define ENTITY_STATIC -2 // Is used for flagging special things in the homesector property.
#define ENTITY_HOMELESS -1 //Entity needs a home.
#define ENTITY_EFFECT -3 //Owned by no sector. Gets rendered last.
#define ENTITY_PRERENDER -4 //Is the first of the entities to be renderes (For the damn sky)

#define ENTITY_MAXANIMATIONS 128
#define ENTITY_MAXCHILDREN 100
#define ENTITY_DEFAULTFRAMEDELAY 100



/*

	En entity skall alltid rkna med GLprops: 
	GL_BLEND = FALSE
	GL_TEXTURE_2D = TRUE


*/


#define ENTITY_TYPE_EMPTY 0


struct Entity_FileStruct
{
	int type;
	int homesector;
	int renderpriority;
	int saveinformationsize;
	int currentanimation;
	int currentframe;
	DWORD framedelay;
	DWORD lastframetick;
	GLVECTOR3f pos;
	GLVECTOR3f dir;
	int children;
};


struct Entity_Event
{
	DWORD tick;
	int currentanimation;
	int currentframe;
	GLVECTOR3f pos;
	GLVECTOR3f dir;
	int impulseindex;
	int impulsevalue;
};

#define ENTITY_RECORD_NOTHING 0
#define ENTITY_RECORD_PLAYING 1
#define ENTITY_RECORD_RECORDING 2


class Entity
{
public:

//Record information;
	int recordmode; 
	DWORD recordstarttick;
	DWORD nextsampletick;
	int sampletime;
	Entity_Event currentevent;
	Entity_Event nextevent;
	GLVECTOR3f currentrecorddata[10];
	GLVECTOR3f nextrecorddata[10];
	ofstream recordfile;
	ifstream playfile;

	GLVECTOR3f *recorddata[10];
	int directional[10];
	int recorddatas;
	int recordimpulse[32];
	void addrecorddata(GLVECTOR3f *trg,int isdirectional = 0) {recorddata[recorddatas] = trg; directional[recorddatas] = isdirectional;recorddatas++;};

	int getnextevent();	
	void sample(DWORD tick);
	void startrecord(char *filename, DWORD starttick,int stime);
	void endrecord();
	void playback(DWORD tick);
	void play(char *filename,DWORD starttick);


//Entity information:
	int type;
	int homesector; //Specifies which sector indice the entity is currently registered into.
	int renderpriority; 

	void *saveinformation;
	int saveinformationsize;
	int *impulse; 

//Model information:
	M3a *model[ENTITY_MAXANIMATIONS];
	int currentanimation;
	int currentframe;
	DWORD framedelay;
	DWORD lastframetick;
	void setanimation(int anim,int frame = 0);
	void setframedelay(DWORD delay) {framedelay = delay;};
	void cycleanimation();

//Position information
	GLVECTOR3f pos;
	GLVECTOR3f dir;

//High-end functions
	int render();
	int move(DWORD tick);
	int defaultrender();
	virtual int renderme();
	virtual int moveme();
	virtual int init() {return(1);}
	virtual int beforesave() {return(1);}; //Do preparing stuff before saving. (Fill the savebuffer, for example?)
	virtual int afterload() {return(1);};  //Do initializing stuff after loading. Will be called AFTER init();
	virtual GLVECTOR3f *getcameraref() {return(&dir);};

	int save(ofstream *fout);
	int load(ifstream *fin);
	void lookat(GLVECTOR3f *p);


//Inheritance
	Entity *child[ENTITY_MAXCHILDREN];
	int children;
	Entity *parent;
	int childpos(Entity *e);

	void setparent(Entity *p) {parent = p;};
	void addchild(Entity *ch);
	void removechild(Entity *ch);


// System
	Entity *bringup[ENTITY_MAXCHILDREN]; // Is a list of entities which the world will collect up to the big entity list.
	int ambringup; //Specifies how many entities to bring up.

	int REMOVEME; // If true, this entity will be killed and removed from world.

	void clear();
	Entity();
	~Entity();

};


void Entity::playback(DWORD tick)
{
	static int num;
	float elaps;
	GLVECTOR3f temp;
	float ftemp;
	while (tick >= nextevent.tick + recordstarttick)
		if (!getnextevent())
		{
			recordmode = ENTITY_RECORD_NOTHING;
			playfile.close();
			return;
		}
	elaps = (float)(tick-(currentevent.tick+recordstarttick)) / (float)(nextevent.tick-currentevent.tick);

	temp = nextevent.pos - currentevent.pos;
	pos = currentevent.pos + temp*elaps;
	temp = nextevent.dir - currentevent.dir;
	dir = currentevent.dir + temp*elaps;

	currentanimation = currentevent.currentanimation;
	currentframe = currentevent.currentframe;
	if (currentevent.impulseindex != -1)
		impulse[currentevent.impulseindex] = currentevent.impulsevalue;

	for (num = 0; num < recorddatas; num++)
	{
		temp = nextrecorddata[num] - currentrecorddata[num];
		if (directional[num])
		{
			if (fabs(nextrecorddata[num].y-currentrecorddata[num].y) < fabs((currentrecorddata[num].y > nextrecorddata[num].y? 360-currentrecorddata[num].y + nextrecorddata[num].y : 360-nextrecorddata[num].y + currentrecorddata[num].y) ) )
				temp.y = nextrecorddata[num].y-currentrecorddata[num].y;
			else
			{
				if (currentrecorddata[num].y > nextrecorddata[num].y)
					temp.y = nextrecorddata[num].y + 360 - currentrecorddata[num].y;
				else
					temp.y = -360+nextrecorddata[num].y - currentrecorddata[num].y;
			}

		}
		*recorddata[num] = currentrecorddata[num] + temp *  elaps;
	}
}

void Entity::play(char *filename,DWORD starttick)
{
	if (recordmode != ENTITY_RECORD_NOTHING) return;
	playfile.open(filename,ios::binary);
	
	getnextevent();
	getnextevent();
	recordstarttick = starttick;
	recordmode = ENTITY_RECORD_PLAYING;
}

int Entity::getnextevent()
{
	static int num;
	if (playfile.eof()) return(0);
	memcpy(&currentevent,&nextevent,sizeof(Entity_Event));
	memcpy(currentrecorddata,&nextrecorddata,sizeof(GLVECTOR3f)*10);

	playfile.read((char*)&nextevent,sizeof(Entity_Event));
	for (num = 0; num < recorddatas; num++)
		playfile.read((char*)&nextrecorddata[num],sizeof(GLVECTOR3f));
	if (playfile.fail()) return(0);
	return(1);
}

void Entity::sample(DWORD tick)
{
	if (recordmode != ENTITY_RECORD_RECORDING) return;
	static int num;
	currentevent.tick = tick - recordstarttick;
	currentevent.currentanimation = currentanimation;
	currentevent.currentframe = currentframe;
	currentevent.dir = dir;
	currentevent.pos = pos;
	currentevent.impulseindex = -1;
	for (num = 0; num < 32; num++)
		if (recordimpulse[num] != impulse[num])
		{
			recordimpulse[num] = impulse[num];
			currentevent.impulseindex = num;
			currentevent.impulsevalue = impulse[num];
			break;	
		}

	recordfile.write((char*)&currentevent,sizeof(Entity_Event));
	for (num = 0; num < recorddatas; num++)
		recordfile.write((char*)recorddata[num],sizeof(GLVECTOR3f));
	nextsampletick += sampletime;
}



void Entity::endrecord()
{
	recordfile.close();
	recordmode = ENTITY_RECORD_NOTHING;
}

void Entity::startrecord(char *filename,DWORD starttickcount,int stime)
{
	int num;
	if (recordmode != ENTITY_RECORD_NOTHING) return;
	recordfile.open(filename,ios::binary | ios::trunc);

	recordstarttick = starttickcount;
	nextsampletick = recordstarttick;
	sampletime = stime;
	recordmode = ENTITY_RECORD_RECORDING;
	for (num = 0; num < 32; num++)
		recordimpulse[num] = 0;
	sample(starttickcount); //Sample here.
}



/*
struct Entity_FileStruct
{
	int type;
	int homesector;
	int renderpriority;
	int saveinformationsize;
	int currentanimation;
	int currentframe;
	DWORD framedelay;
	DWORD lastframetick;
	GLVECTOR3f pos;
	GLVECTOR3f dir;
	int children;
};
*/

int Entity::childpos(Entity *e)
{
	int num;
	for (num = 0; num < children; num++)
	{
		if (child[num] = e) return(num);
	}
	return(-1);
}

int Entity::load(ifstream *fin)
{
	Entity_FileStruct h;
	fin->read((char*)&h,sizeof(h));
	if (type != h.type) 
	{
		MessageBox(NULL,"Incompatible entity type on load!","Error",MB_OK);
		return(0);
	}
//	type = h.type;
	homesector = h.homesector;
	renderpriority = h.renderpriority;
	saveinformationsize = h.saveinformationsize;
	currentanimation = h.currentanimation;
	currentframe = h.currentframe;
	framedelay = h.framedelay;
	lastframetick = GetTickCount() - h.lastframetick;
	pos = h.pos;
	dir = h.dir;
	children = h.children;
	

	if (saveinformationsize)
	{
		saveinformation = new char[saveinformationsize];
		fin->read((char*)saveinformation,saveinformationsize);
	}


	if (homesector > ENTITY_HOMELESS) homesector = ENTITY_HOMELESS;

	int num;
	//TODO: Load algorithm for Children
	for (num = 0; num < children; num++)
	{
		
	}

	return(1);
}


int Entity::save(ofstream *fout)
{
	beforesave(); 


	Entity_FileStruct h;
	h.type = type;
	h.homesector = homesector;
	h.renderpriority = renderpriority;
	h.saveinformationsize = saveinformationsize;
	h.currentanimation = currentanimation;
	h.currentframe = currentframe;
	h.framedelay = framedelay;
	h.lastframetick = GetTickCount() - lastframetick;
	h.pos = pos;
	h.dir = dir;
	h.children = children;

	fout->write((char*)&h,sizeof(h));
	if (saveinformationsize)
		fout->write((char*)saveinformation,saveinformationsize);
	int num;

	//TODO: Save algorithm for children
	for (num = 0; num < children; num++) //Save the children!
	{
	}
	if (saveinformationsize)
	{
		delete[] saveinformation;
		saveinformationsize = 0;
	}
	return(1);
}


void Entity::clear()
{
	int num;
	for (num = 0; num < ENTITY_MAXANIMATIONS; num++)
		if (model[num])
		{
			delete model[num];
			model[num] = NULL;
		}
	if (!parent)
		if (impulse)
			delete[] impulse;
	for (num = 0; num < children; num++)
		delete child[num];

}

Entity::~Entity()
{
	clear();
}


int Entity::defaultrender()
{
	if (model[currentanimation] && model[currentanimation]->animations > currentframe)
		model[currentanimation]->model[currentframe].render();
	return(1);
}


void Entity::removechild(Entity *ch)
{
	static int num,num2;
	for (num = 0; num < children; num++)
	{
		if (child[num] == ch) 
		{
			delete child[num];
			children--;
			for (num2 = num; num2 < children; num2++)
			{
				child[num2] = child[num2+1];
			}
		}
	}
}


void Entity::addchild(Entity *ch)
{
	child[children] = ch;
	if (ch->impulse)
		delete[] ch->impulse; 
	ch->impulse = impulse; //Share the impulse.
	ch->parent = this;
	ch->init();
	bringup[ambringup] = ch;
	ambringup++;
//	currentworld->loadentitytextures(ch);
	children++;
}



int Entity::renderme()
{
	return(defaultrender());
}

int Entity::move(DWORD tick)
{

	int num;



	if (recordmode == ENTITY_RECORD_RECORDING)
	{
		while (tick >= nextsampletick)
			sample(tick);
		for (num = 0; num < 32; num++)
			if (recordimpulse[num] != impulse[num]) sample(tick);
	}
	
	if (recordmode == ENTITY_RECORD_PLAYING)
	{
		playback(tick);
	}
	else
		moveme();




	for (num = 0; num < children; num++)
		child[num]->move(tick);


	return(1);
}

int Entity::render()
{
	glPushMatrix();

	glTranslatef(pos.x,pos.y,pos.z);
	glRotatef(-dir.y,0,1,0);
	glRotatef(-dir.x,1,0,0);
	glRotatef(-dir.z,0,0,1);

	int num;
	cycleanimation();
	renderme();
	for (num = 0; num < children; num++)
		child[num]->render();

	glPopMatrix();
	return(1);
}

int Entity::moveme()
{
	return(-1);
}

Entity::Entity()
{
	int num;
	for (num = 0; num < ENTITY_MAXANIMATIONS; num++)
		model[num] = NULL;

	impulse = new int[32];
	for (num = 0; num < 32; num++)
		impulse[num] = 0;
	parent = NULL;
	children = 0;
	framedelay = ENTITY_DEFAULTFRAMEDELAY;
	setanimation(0,0);
	dir.set();
	pos.set();
	renderpriority = PRIORITY_NORMAL;
	type = ENTITY_TYPE_EMPTY;
	homesector = ENTITY_HOMELESS;
	ambringup = 0;
	REMOVEME = 0;
	saveinformationsize = 0;
	recordmode = ENTITY_RECORD_NOTHING;
	recorddatas = 0;
}

void Entity::cycleanimation()
{	
	if (GetTickCount() >= lastframetick + framedelay)
	{
		lastframetick = GetTickCount();
		currentframe++;
		
		if (!model[currentanimation] || currentframe >= model[currentanimation]->animations)
			currentframe = 0;
	}
}

void Entity::setanimation(int anim,int frame)
{
	currentanimation = anim;
	currentframe = frame;
	lastframetick = GetTickCount();
}



#endif