#ifndef __M3A__
#define __M3A__

//Lyes above the .m3o file format. Combines many .m3o-files, thus making an animation.


#include "mystuff/m3o.h"

#define M3A_MAXANIMS 1024
#define M3A_VERSION 100


struct M3aFileHeader
{
	WORD version;
	int animations;
};




class M3a
{
public:
	M3o model[M3A_MAXANIMS];
	int animations;

	void import(int index,M3o *src);
	void readobjs(char *file,int end,int start = 0);
	void load(char *fil);
	void save(char *fil);
	M3a();
};

M3a::M3a()
{
	animations = 0;
}

void M3a::save(char *fil)
{
	ofstream fout;

	fout.open(fil,ios::binary | ios::trunc);

	M3aFileHeader fh;
	fh.version = M3A_VERSION;
	fh.animations = animations;

	fout.write((char*)&fh,sizeof(fh));

	int num;
	for (num = 0; num < animations; num++)
	{
		model[num]._streamsave(&fout);
	}


	fout.close();
}

void M3a::load(char *fil)
{
	ifstream fin;

	fin.open(fil,ios::binary);

	M3aFileHeader fh;

	fin.read((char*)&fh,sizeof(fh));

	if (fh.version != M3A_VERSION) return;
	animations = fh.animations;

	int num;
	for (num = 0; num < animations; num++)
	{
		model[num]._streamload(&fin);
	}


	fin.close();
	
}

void M3a::readobjs(char *file,int end,int start)
{
	int num;
	char fil[128];
	M3o m3o;
	for (num = start; num < end; num++)
	{
		sprintf(fil,"%s%i.obj",file,num);		
		m3o.loadOBJ(fil);
		import(num-start,&m3o);			
	}
}


void M3a::import(int index,M3o *src)
{
	if (index >= animations) 
	{
		if (index > animations)
			index = animations;
		animations++;
	}

	model[index].vertex = new GLVECTOR3f[src->vertexes];
	memcpy(model[index].vertex,src->vertex,src->vertexes*sizeof(GLVECTOR3f));
	model[index].vertexes = src->vertexes;

	model[index].normal = new GLVECTOR3f[src->normals];
	memcpy(model[index].normal,src->normal,src->normals*sizeof(GLVECTOR3f));
	model[index].normals = src->normals;

	model[index].texcoord = new GLVECTOR2f[src->texcoords];
	memcpy(model[index].texcoord,src->texcoord,src->texcoords*sizeof(GLVECTOR2f));
	model[index].texcoords = src->texcoords;

	model[index].material = new M3oMaterial[src->materials];
	memcpy(model[index].material,src->material,src->materials*sizeof(M3oMaterial));
	model[index].materials = src->materials;

	model[index].face = new M3oFace[src->faces];
	memcpy(model[index].face,src->face,src->faces*sizeof(M3oFace));
	model[index].faces = src->faces;

}


#endif