#ifndef __M3O__
#define __M3O__


#define M3OVERSION 100

#include "mystuff/glvector2f.h"
#include "mystuff/glvector3f.h"
#include <fstream.h>
#include "mystuff/debug.h"
#include "mystuff/mpc.h"

//Ka = ambient
//Kd = diffuse
//Ks = Specular
//Ns = Specular highlight
//Kda = diffuse alpha (Syntax: Kda <float> )
//bl = Blend func (Syntax: bl <src> <dst>)


struct M3oFileHeader
{
	DWORD version;
	WORD vertexes;
	WORD normals;
	WORD texcoords;
	WORD faces;
	WORD materials;
};

struct M3oMaterial
{
	unsigned char name[50];
	unsigned char texturename[256];
	GLVECTOR3f ambient;
	GLVECTOR3f diffuse;
	float diffusealpha;
	GLVECTOR3f specular;
	float specularhighlight;
	GLuint texture;
	int doblend;
	GLenum blendsrc,blenddst;
};


#define M3OFACE_HASVERTICE 1
#define M3OFACE_HASNORMAL 2
#define M3OFACE_HASTEXCOORD 4
#define M3OFACE_HASMATERIAL 8

struct M3oFace
{
	int vertice[3];
	int normal[3];
	int texcoord[3];
	int materialID;
	WORD flags;
};

#define M3OIDENT_VERTEX 1
#define M3OIDENT_NORMAL 2
#define M3OIDENT_TEXCOORD 3
#define M3OIDENT_FACE 4
#define M3OIDENT_COMMENT 5
#define M3OIDENT_USEMATERIAL 6
#define M3OIDENT_MAT_SH 30
#define M3OIDENT_MAT_AMBIENT 31
#define M3OIDENT_MAT_DIFFUSE 32
#define M3OIDENT_MAT_SPECULAR 33
#define M3OIDENT_MAT_TEXTURE 34
#define M3OIDENT_MAT_NEWMATERIAL 35
#define M3OIDENT_MAT_BLEND 36
#define M3OIDENT_MAT_DIFFUSEALPHA 37

#define M3OIDENT_UNKNOWN 100

class M3o
{
private:
	int findduptexture(unsigned char *name,int range);
	int getmaterial(unsigned char *str);
	int startstr(unsigned char *buf,char *start);
	int identify_line(unsigned char *str);
	int countspaces(unsigned char *buffer);

	//Returns the amount of <objects>, or -1 if an error occured
	int obj_countelements(char *fil,int ident);
	int obj_countfaces(char *fil); //This one is a bit special.

	//Return 1 if ok, 0 on error
	int obj_readvertex(unsigned char *buffer,int index);
	int obj_readblend(unsigned char *buffer,GLuint *src,GLuint *dst);
	int obj_readnormal(unsigned char *buffer,int index);
	int obj_readtexcoord(unsigned char *buffer,int index);
	int obj_readface(unsigned char *buffer,int materialID,int index,int secondindex = -1);
	int obj_readalpha(unsigned char *buffer,float *alpha);

	int obj_readinteger(unsigned char *buffer);
	int obj_readstring(unsigned char *buffer,unsigned char *target);
	int obj_readcolor(unsigned char *buffer, GLVECTOR3f *color);


	


	int obj_readmaterials(char *fil);

public:
	GLVECTOR3f *vertex;
	int vertexes;
	GLVECTOR3f *normal;
	int normals;
	GLVECTOR2f *texcoord;
	int texcoords;
	M3oMaterial *material;
	int materials;
	M3oFace *face;
	int faces;

	int _streamload(ifstream *fin);
	int _streamsave(ofstream *fout);
	int load(char *filename);
	int save(char *filename);
	int loadOBJ(char *filename);

	void flip(int elem1,int elem2);

	void dumpmaterial(char *fil);
	void dumpdata(char *fil);
	void multiplyvertex(float x,float y,float z);
	void multiplynormal(float x,float y,float z);

	void loadtextures();


	void render();

	M3o();
	~M3o();
};


void M3o::flip(int elem1,int elem2)
{
	int num;
	float temp;
	for (num = 0; num < vertexes; num++)
	{
		temp = vertex[num].getelement(elem1);
		vertex[num].setelement(elem1,vertex[num].getelement(elem2));
		vertex[num].setelement(elem2,temp);
		temp = normal[num].getelement(elem1);
		normal[num].setelement(elem1,normal[num].getelement(elem2));
		normal[num].setelement(elem2,temp);
	}
}

int M3o::_streamsave(ofstream *fout)
{

	M3oFileHeader header;
	int num;
	header.version = M3OVERSION;
	header.faces = faces;
	header.materials = materials;
	header.normals = normals;
	header.texcoords = texcoords;
	header.vertexes = vertexes;

	fout->write((char*)&header,sizeof(header));
	
	for (num = 0; num < vertexes; num++)
	{
		fout->write((char*)&vertex[num],sizeof(GLVECTOR3f));
	}
	for (num = 0; num < normals; num++)
	{
		fout->write((char*)&normal[num],sizeof(GLVECTOR3f));
	}
	for (num = 0; num < texcoords; num++)
	{
		fout->write((char*)&texcoord[num],sizeof(GLVECTOR2f));
	}
	for (num = 0; num < faces; num++)
	{
		fout->write((char*)&face[num],sizeof(M3oFace));
	}
	for (num = 0; num < materials; num++)
	{
		fout->write((char*)&material[num],sizeof(M3oMaterial));
	}
	return(1);
}

int M3o::save(char *filename)
{
	ofstream fout;
	fout.open(filename,ios::binary | ios::trunc);
	

	int ret = _streamsave(&fout);


	fout.close();
	return(ret);

}

int M3o::_streamload(ifstream *fin)
{
	M3oFileHeader header;
	int num;
	fin->read((char*)&header,sizeof(header));

	if (header.version != M3OVERSION) return(0);

	faces = header.faces;
	materials = header.materials;
	normals = header.normals;
	texcoords = header.texcoords;
	vertexes = header.vertexes;


	if (vertex) delete[] vertex;
	vertex = new GLVECTOR3f[vertexes];

	if (normal) delete[] normal;
	normal = new GLVECTOR3f[normals];
	
	if (texcoord) delete[] texcoord;
	texcoord = new GLVECTOR2f[texcoords];

	if (face) delete[] face;
	face = new M3oFace[faces];

	if (material) delete[] material;
	material = new M3oMaterial[materials];


	for (num = 0; num < vertexes; num++)
	{
		fin->read((char*)&vertex[num],sizeof(GLVECTOR3f));
	}
	for (num = 0; num < normals; num++)
	{
		fin->read((char*)&normal[num],sizeof(GLVECTOR3f));
	}
	for (num = 0; num < texcoords; num++)
	{
		fin->read((char*)&texcoord[num],sizeof(GLVECTOR2f));
	}
	for (num = 0; num < faces; num++)
	{
		fin->read((char*)&face[num],sizeof(M3oFace));
	}
	for (num = 0; num < materials; num++)
	{
		fin->read((char*)&material[num],sizeof(M3oMaterial));
	}
	return(1);

}

int M3o::load(char *filename)
{
	ifstream fin;
	fin.open(filename,ios::binary);
	

	int ret = _streamload(&fin);



	fin.close();
	return(ret);

}





void M3o::dumpdata(char *fil)
{
	DebugClass debug;

	debug.open(fil,0);
	debug.put("Vertexes: ");
	debug.put(vertexes);
	debug.put("\nFaces: ");
	debug.put(faces);


	debug.close();
}


int M3o::findduptexture(unsigned char *name,int range)
{
	static int num;
	for (num = 0; num < range; num++)
	{
		if (!strcmp((char*)name,(char*)material[num].texturename)) return(num);
	}
	return(-1);
}

void M3o::multiplyvertex(float x,float y,float z)
{
	static int num; 

	for (num = 0; num < vertexes; num++)
	{
		vertex[num].x *= x;
		vertex[num].y *= y;
		vertex[num].z *= z;
	}
}

void M3o::multiplynormal(float x,float y,float z)
{
	static int num; 

	for (num = 0; num < vertexes; num++)
	{
		normal[num].x *= x;
		normal[num].y *= y;
		normal[num].z *= z;
	}
}

void M3o::render()
{
#define M3ORENDERMODE GL_TRIANGLES
	static int num;
	static GLuint curmat;
	static float color[4] = {0,0,0,1};
	curmat = -1;
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
//	glBindTexture(GL_TEXTURE_2D,material[curmat].texture);
	glBegin(M3ORENDERMODE);
	for (num = 0; num < faces; num++)
	{

		if (curmat != face[num].materialID)
		{
			curmat = face[num].materialID;
			glEnd();
			if (material[curmat].texture != -1)
			{
				glEnable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D,material[curmat].texture);
			}
			else glDisable(GL_TEXTURE_2D);
			color[0] = 1;
			color[1] = 1;
			color[2] = 1;
			color[3] = 1;
			glLightModelfv(GL_LIGHT_MODEL_AMBIENT,color);

			color[0] = material[curmat].ambient.x;
			color[1] = material[curmat].ambient.y;
			color[2] = material[curmat].ambient.z;
			color[3] = 1;
			glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,color);
//			glLightfv(GL_LIGHT0,GL_AMBIENT,color);
			color[0] = material[curmat].diffuse.x;
			color[1] = material[curmat].diffuse.y;
			color[2] = material[curmat].diffuse.z;
			color[3] = 1;

			glLightfv(GL_LIGHT0,GL_DIFFUSE,color);
			color[0] = material[curmat].specular.x;
			color[1] = material[curmat].specular.y;
			color[2] = material[curmat].specular.z;
			color[3] = 1;
			glLightfv(GL_LIGHT0,GL_SPECULAR,color);

//			glColor4fv((float*)&material[curmat].diffuse);
			
			if (material[curmat].doblend)
			{
				glEnable(GL_BLEND);
				glBlendFunc(material[curmat].blendsrc,material[curmat].blenddst);
			}
			else glDisable(GL_BLEND);

			glBegin(M3ORENDERMODE);
		}
		glNormal3fv((float*)&normal[face[num].normal[0]]);
		glTexCoord2fv((float*)&texcoord[face[num].texcoord[0]]);
		glVertex3fv((float*)&vertex[face[num].vertice[0]]);
		glNormal3fv((float*)&normal[face[num].normal[1]]);
		glTexCoord2fv((float*)&texcoord[face[num].texcoord[1]]);
		glVertex3fv((float*)&vertex[face[num].vertice[1]]);
		glNormal3fv((float*)&normal[face[num].normal[2]]);
		glTexCoord2fv((float*)&texcoord[face[num].texcoord[2]]);
		glVertex3fv((float*)&vertex[face[num].vertice[2]]);
	}
	glEnd();
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
	glDisable(GL_LIGHTING);
}


void M3o::loadtextures()
{
	Mpc mpc;

	int num;
	int tex; 
	for (num = 0; num < materials; num++)
	{
		tex = findduptexture(material[num].texturename,num);
		if (tex == -1)
		{
			if (material[num].texturename[0] != 0)
			{
				mpc.loadbmp((char*)material[num].texturename);
				// mpc.flipplane(0,2); This bug is now fixed.
				material[num].texture = mpc.createtexture(2);
				mpc.clear();
			}
			else
				material[num].texture = -1;
		}
		else material[num].texture = material[tex].texture;
	}

}

int M3o::countspaces(unsigned char *buffer)
{
	static int pos;
	static int amount;
	amount = 0;
	pos = 0;
	while(buffer[pos] != 0)
	{
		if (buffer[pos] == ' ') amount++;
		pos++;
	}
	return(amount);
}


int M3o::getmaterial(unsigned char *str)
{
	static int num;
	for (num = 0; num < materials; num++)
	{
		if (!strcmp((char*)str,(char*)material[num].name)) return(num);
	}
	return(-1);
}

int M3o::loadOBJ(char *filename)
{
	char fil[256];
	unsigned char buffer[1024];
	strcpy(fil,filename);
	strcat(fil,".mtl");
	obj_readmaterials(fil);
	strcpy(fil,filename);
	strcat(fil,".obj");
	ifstream fin;


	int type;

	if (vertex) delete[] vertex;
	vertexes = obj_countelements(fil,M3OIDENT_VERTEX);
	vertex = new GLVECTOR3f[vertexes];

	if (normal) delete[] normal;
	normals = obj_countelements(fil,M3OIDENT_NORMAL);
	normal = new GLVECTOR3f[normals];
	
	if (texcoord) delete[] texcoord;
	texcoords = obj_countelements(fil,M3OIDENT_TEXCOORD);
	texcoord = new GLVECTOR2f[texcoords];

	if (face) delete[] face;
	faces = obj_countfaces(fil);
	face = new M3oFace[faces];


	vertexes = 0;
	normals = 0;
	texcoords = 0;
	faces = 0;


	fin.open(fil);
	if (fin.fail()) return(0);

	int currmaterial = -1;
	int temp;
	unsigned char tempbuf[100];

	while (!fin.eof())
	{
		fin.getline(buffer,1024);
		type = identify_line(buffer);
//#define M3OIDENT_VERTEX 1
//#define M3OIDENT_NORMAL 2
//#define M3OIDENT_TEXCOORD 3
//#define M3OIDENT_FACE 4
//#define M3OIDENT_COMMENT 5
//#define M3OIDENT_USEMATERIAL 6

		switch(type)
		{
		case M3OIDENT_VERTEX:
			obj_readvertex(buffer,vertexes);
			vertexes++;
		break;
		case M3OIDENT_NORMAL:
			obj_readnormal(buffer,normals);
			normals++;
		break;
		case M3OIDENT_TEXCOORD:
			obj_readtexcoord(buffer,texcoords);
			texcoords++;
		break;
		case M3OIDENT_FACE:
			temp = obj_readface(buffer,currmaterial,faces);
			faces += temp;
		break;
		case M3OIDENT_COMMENT:
		break;
		case M3OIDENT_USEMATERIAL:
			obj_readstring(buffer,tempbuf);
			currmaterial = getmaterial(tempbuf);
		break;


		}

	}

	if (vertexes != obj_countelements(fil,M3OIDENT_VERTEX))
		MessageBox(NULL,"Vertex count mismatch!!","",MB_OK);
	if (normals != obj_countelements(fil,M3OIDENT_NORMAL))
		MessageBox(NULL,"Normal count mismatch!!","",MB_OK);
	if (texcoords != obj_countelements(fil,M3OIDENT_TEXCOORD))
		MessageBox(NULL,"texcoord count mismatch!!","",MB_OK);
	if (faces != obj_countfaces(fil))
		MessageBox(NULL,"Face count mismatch!!","",MB_OK);


	fin.close();

	return(1);
}

void M3o::dumpmaterial(char *fil)
{
	DebugClass debug;

	debug.open(fil,0);
	int index;
	for (index = 0; index < materials; index++)
	{
		debug.put("--------------------\nID: ");
		debug.put(index);
		debug.put("\nName: ");
		debug.put((char*)material[index].name);
		debug.put("\nAmbient: ");
		debug.put(material[index].ambient.x);
		debug.put(", ");
		debug.put(material[index].ambient.y);
		debug.put(", ");
		debug.put(material[index].ambient.z);
		debug.put("\nDiffuse: ");
		debug.put(material[index].diffuse.x);
		debug.put(", ");
		debug.put(material[index].diffuse.y);
		debug.put(", ");
		debug.put(material[index].diffuse.z);
		debug.put(", ");
		debug.put(material[index].diffusealpha);
		debug.put("\nSpecular: ");
		debug.put(material[index].specular.x);
		debug.put(", ");
		debug.put(material[index].specular.y);
		debug.put(", ");
		debug.put(material[index].specular.z);
		debug.put("\nHighlight: ");
		debug.put(material[index].specularhighlight);
		debug.put("\nTexture: ");
		debug.put((char*)material[index].texturename);
		debug.put("\n");
		if (material[index].doblend)
		{
			debug.put("BlendFunc: ");
			debug.put((int)material[index].blendsrc);
			debug.put(", ");
			debug.put((int)material[index].blenddst);
			debug.put("\n");
		}
	}

	debug.close();
	
}

int M3o::obj_readcolor(unsigned char *buffer,GLVECTOR3f *color)
{
	static int pos;
	pos = 0;
	while (buffer[pos] != ' ') pos++;
	pos++;
	sscanf((char*)&buffer[pos],"%e %e %e",&color->x,&color->y,&color->z);
	return(1);
}

int M3o::obj_readinteger(unsigned char *buffer)
{
	static int pos;
	pos = 0;
	while (buffer[pos] != ' ') pos++;
	pos++;
	return(atoi((char*)&buffer[pos]));
}


int M3o::obj_readstring(unsigned char *buffer, unsigned char *target)
{
	static int pos,pos2;

	pos = 0;
	pos2 = 0;
	while (buffer[pos] != ' ') pos++;
	pos++;
	while (buffer[pos])
	{
		target[pos2] = buffer[pos];
		pos++;
		pos2++;
	}
	target[pos2] = 0;
	return(1);
}




M3o::M3o()
{
	vertex = NULL;
	normal = NULL;
	texcoord = NULL;
	material = NULL;
	face = NULL;
}

M3o::~M3o()
{
	if (vertex) delete[] vertex;
	if (normal) delete[] normal;
	if (texcoord) delete[] texcoord;
	if (material) delete[] material;
	if (face) delete[] face;
}



int M3o::obj_readmaterials(char *fil)
{
	ifstream fin;

	materials = obj_countelements(fil,M3OIDENT_MAT_NEWMATERIAL);
	if (material) delete[] material; 
	material = new M3oMaterial[materials];

	int index;
	index = -1;

	unsigned char buffer[1024];
	int type;

	fin.open(fil);
	while (!fin.eof())
	{
		fin.getline(buffer,1024);
		type = identify_line(buffer);

		
		switch(type)
		{
		case M3OIDENT_MAT_SH:
			material[index].specularhighlight = obj_readinteger(buffer);
		break;			
		case M3OIDENT_MAT_AMBIENT: 
			obj_readcolor(buffer,&material[index].ambient);
		break;			
		case M3OIDENT_MAT_DIFFUSE: 
			obj_readcolor(buffer,&material[index].diffuse);
		break;			
		case M3OIDENT_MAT_SPECULAR: 
			obj_readcolor(buffer,&material[index].specular);
		break;			
		case M3OIDENT_MAT_TEXTURE: 
			obj_readstring(buffer,material[index].texturename);
		break;			
		case M3OIDENT_MAT_BLEND:
			obj_readblend(buffer,&material[index].blendsrc,&material[index].blenddst);			
			material[index].doblend = 1;
		break;
		case M3OIDENT_MAT_DIFFUSEALPHA:
			obj_readalpha(buffer,&material[index].diffusealpha);
		break;
		case M3OIDENT_MAT_NEWMATERIAL: 
			index++;
			obj_readstring(buffer,material[index].name);
			material[index].texturename[0] = 0;
			material[index].texture = -1;
			material[index].ambient.x = 0;
			material[index].ambient.y = 0;
			material[index].ambient.z = 0;
			material[index].diffuse.x = 1;
			material[index].diffuse.y = 1;
			material[index].diffuse.z = 1;
			material[index].specular.x = 0;
			material[index].specular.y = 0;
			material[index].specular.z = 0;
			material[index].diffusealpha = 1.0f;
			material[index].doblend = 0;
		break;			

		}
	}
	fin.close();
	return(1);
}

int M3o::startstr(unsigned char *buf,char *start)
{
	static unsigned char *s;
	s = (unsigned char*)start;
	static int pos;
	pos = 0;
	while (buf[pos] != 0 && s[pos] != 0 && buf[pos] == s[pos]) pos++;

	if (s[pos] == 0) return(1);

	return(0);

}

int M3o::identify_line(unsigned char *str)
{
	if (startstr(str,"v ")) return(M3OIDENT_VERTEX);
	if (startstr(str,"vn ")) return(M3OIDENT_NORMAL);
	if (startstr(str,"vt ")) return(M3OIDENT_TEXCOORD);
	if (startstr(str,"f ")) return(M3OIDENT_FACE);
	if (startstr(str,"#")) return(M3OIDENT_COMMENT);
	if (startstr(str,"usemtl ")) return(M3OIDENT_USEMATERIAL);
	if (startstr(str,"newmtl ")) return(M3OIDENT_MAT_NEWMATERIAL);
	if (startstr(str,"Ns ")) return(M3OIDENT_MAT_SH);
	if (startstr(str,"Ka ")) return(M3OIDENT_MAT_AMBIENT);
	if (startstr(str,"Kd")) return(M3OIDENT_MAT_DIFFUSE);
	if (startstr(str,"Ks ")) return(M3OIDENT_MAT_SPECULAR);
	if (startstr(str,"map_Kd ")) return(M3OIDENT_MAT_TEXTURE);
	if (startstr(str,"bl ")) return(M3OIDENT_MAT_BLEND);
	if (startstr(str,"kda ")) return(M3OIDENT_MAT_DIFFUSEALPHA);

	return(M3OIDENT_UNKNOWN);

}


int M3o::obj_readalpha(unsigned char *buffer,float *alpha)
{
	sscanf((char*)buffer,"Kda %f",alpha);
	return(1);
}

int M3o::obj_readvertex(unsigned char *buffer,int index)
{
	sscanf((char*)buffer,"v %e %e %e",&vertex[index].x,&vertex[index].y,&vertex[index].z);
	return(1);
}

int M3o::obj_readnormal(unsigned char *buffer,int index)
{
	sscanf((char*)buffer,"vn %e %e %e",&normal[index].x,&normal[index].y,&normal[index].z);
	return(1);
}

int M3o::obj_readblend(unsigned char *buffer, GLuint *src,GLuint *dst)
{
	static char str[2][80];
	sscanf((char*)buffer,"bl %s %s",str[0],str[1]);
	if (!strcmp(str[0],"GL_ONE")) *src = GL_ONE;
	else if (!strcmp(str[0],"GL_ZERO")) *src = GL_ZERO;
	else if (!strcmp(str[0],"GL_SRC_COLOR")) *src = GL_SRC_COLOR;
	else if (!strcmp(str[0],"GL_ONE_MINUS_SRC_COLOR")) *src = GL_ONE_MINUS_SRC_COLOR;
	else if (!strcmp(str[0],"GL_DST_COLOR")) *src = GL_DST_COLOR;
	else if (!strcmp(str[0],"GL_ONE_MINUS_DST_COLOR")) *src = GL_ONE_MINUS_DST_COLOR;
	else if (!strcmp(str[0],"GL_SRC_ALPHA")) *src = GL_SRC_ALPHA;
	else if (!strcmp(str[0],"GL_ONE_MINUS_SRC_ALPHA")) *src = GL_ONE_MINUS_SRC_ALPHA;
	else if (!strcmp(str[0],"GL_SRC_ALPHA_SATURATE")) *src = GL_SRC_ALPHA_SATURATE;
	else MessageBox(NULL,"Incorrect SRC blend function.","ObjLoad ERROR",MB_OK);

	if (!strcmp(str[1],"GL_ONE")) *dst = GL_ONE;
	else if (!strcmp(str[1],"GL_ZERO")) *dst = GL_ZERO;
	else if (!strcmp(str[1],"GL_SRC_COLOR")) *dst = GL_SRC_COLOR;
	else if (!strcmp(str[1],"GL_ONE_MINUS_SRC_COLOR")) *dst = GL_ONE_MINUS_SRC_COLOR;
	else if (!strcmp(str[1],"GL_DST_COLOR")) *dst = GL_DST_COLOR;
	else if (!strcmp(str[1],"GL_ONE_MINUS_DST_COLOR")) *dst = GL_ONE_MINUS_DST_COLOR;
	else if (!strcmp(str[1],"GL_SRC_ALPHA")) *dst = GL_SRC_ALPHA;
	else if (!strcmp(str[1],"GL_ONE_MINUS_SRC_ALPHA")) *dst = GL_ONE_MINUS_SRC_ALPHA;
	else if (!strcmp(str[1],"GL_SRC_ALPHA_SATURATE")) *dst = GL_SRC_ALPHA_SATURATE;
	else MessageBox(NULL,"Incorrect DST blend function.","ObjLoad ERROR",MB_OK);


	return(1);
}

int M3o::obj_readtexcoord(unsigned char *buffer,int index)
{
	sscanf((char*)buffer,"vt %e %e",&texcoord[index].x,&texcoord[index].y);
	return(1);
}

int M3o::obj_readface(unsigned char *buffer,int materialID,int index,int secondindex)
{
	int f_vertice[4];
	int f_texcoord[4];
	int f_normal[4];
	int goodindex;

	static int amount;
	
	amount = countspaces(buffer);

	if (amount == 3) //a triangle
	{
		sscanf((char*)buffer,"f %d/%d/%d %d/%d/%d %d/%d/%d",&f_vertice[0],&f_texcoord[0],&f_normal[0],&f_vertice[1],&f_texcoord[1],&f_normal[1],&f_vertice[2],&f_texcoord[2],&f_normal[2]);
		if (materialID != -1) 
		{
			face[index].materialID = materialID;
			face[index].flags = M3OFACE_HASMATERIAL;
		} else face[index].flags = 0;
		face[index].flags = face[index].flags | M3OFACE_HASVERTICE | M3OFACE_HASTEXCOORD | M3OFACE_HASNORMAL;
		face[index].vertice[0] = f_vertice[0]-1;
		face[index].vertice[1] = f_vertice[1]-1;
		face[index].vertice[2] = f_vertice[2]-1;
		face[index].normal[0] = f_normal[0]-1;
		face[index].normal[1] = f_normal[1]-1;
		face[index].normal[2] = f_normal[2]-1;
		face[index].texcoord[0] = f_texcoord[0]-1;
		face[index].texcoord[1] = f_texcoord[1]-1;
		face[index].texcoord[2] = f_texcoord[2]-1;
		return(1);

	}
	if (amount == 4) //a quad
	{
		sscanf((char*)buffer,"f %d/%d/%d %d/%d/%d %d/%d/%d %d/%d/%d",&f_vertice[0],&f_texcoord[0],&f_normal[0],&f_vertice[1],&f_texcoord[1],&f_normal[1],&f_vertice[2],&f_texcoord[2],&f_normal[2],&f_vertice[3],&f_texcoord[3],&f_normal[3]);
		if (secondindex == -1)
			goodindex = index+1;
		else goodindex = secondindex;

		if (materialID != -1) 
		{
			face[index].materialID = materialID;
			face[index].flags = M3OFACE_HASMATERIAL;
			face[goodindex].materialID = materialID;
			face[goodindex].flags = M3OFACE_HASMATERIAL;
		} 
		else 
		{
			face[index].flags = 0;
			face[goodindex].flags = 0;
		}
		face[index].flags = face[index].flags | M3OFACE_HASVERTICE | M3OFACE_HASTEXCOORD | M3OFACE_HASNORMAL;
		face[goodindex].flags = face[goodindex].flags | M3OFACE_HASVERTICE | M3OFACE_HASTEXCOORD | M3OFACE_HASNORMAL;

		face[index].vertice[0] = f_vertice[0]-1;
		face[index].vertice[1] = f_vertice[1]-1;
		face[index].vertice[2] = f_vertice[2]-1;
		face[index].normal[0] = f_normal[0]-1;
		face[index].normal[1] = f_normal[1]-1;
		face[index].normal[2] = f_normal[2]-1;
		face[index].texcoord[0] = f_texcoord[0]-1;
		face[index].texcoord[1] = f_texcoord[1]-1;
		face[index].texcoord[2] = f_texcoord[2]-1;

		
		face[goodindex].vertice[0] = f_vertice[0]-1;
		face[goodindex].vertice[1] = f_vertice[2]-1;
		face[goodindex].vertice[2] = f_vertice[3]-1;
		face[goodindex].normal[0] = f_normal[0]-1;
		face[goodindex].normal[1] = f_normal[2]-1;
		face[goodindex].normal[2] = f_normal[3]-1;
		face[goodindex].texcoord[0] = f_texcoord[0]-1;
		face[goodindex].texcoord[1] = f_texcoord[2]-1;
		face[goodindex].texcoord[2] = f_texcoord[3]-1;


		
		return(2);

	}


	return(0);
}


int M3o::obj_countfaces(char *fil)
{
	int count = 0;
	int temp;
	unsigned char buffer[1024];
	ifstream fin;
	
	fin.open(fil);
	while (!fin.eof())
	{
		fin.getline(buffer,1024);
		if (identify_line(buffer) == M3OIDENT_FACE) 
		{
			temp = countspaces(buffer);
			if (temp == 3) count++;
			if (temp == 4) count+=2;
		}

	}
	fin.close();

	return(count);
}


int M3o::obj_countelements(char *fil,int ident)
{
	int count = 0;
	unsigned char buffer[1024];
	ifstream fin;
	
	fin.open(fil);
	while (!fin.eof())
	{
		fin.getline(buffer,1024);
		if (identify_line(buffer) == ident) count++;
	}
	fin.close();

	return(count);
}



#endif