#ifndef __MPC__
#define __MPC__

/*
	Is the .MPC image file format handler.
*/

#include <fstream.h>

#define MPC_CURRENTVERSION 200

struct MPC_VERSIONHEADER
{
	WORD version;
};

struct MPC_HEADER
{
	WORD xsize;
	WORD ysize;
	WORD bpp;
	WORD reserved[7];
};


class Mpc
{
public:
	GLuint texture;
	unsigned char *pixels;
	MPC_HEADER header;

	int load(char *filnamn); //Loads a .MPC file
	int save(char *filnamn); //Loads a .BMP file

	int savebmp(char *filnamn); //Saves the file in .BMP format.

	int loadbmp(char *filnamn); //Imports a .BMP file

	//Loads two .BMP files. the first for RGB-planes, and the second for the
	//alpha plane. (plane tells which plane to read. 0 = red,1 = green, 2 = blue
	int loadbmp4(char *filnamn,char *filnamn2,int plane = 0); 
	//Loads a .BMP file and tells which color to set as transparent.
	int loadbmpwithmask(char *filnamn,unsigned char r,unsigned char g,unsigned char b);

	//Dumps the image to a text file (Only for debug purposes)
	void dumpimage(char *file);

	//Creates a texture out of the image
	GLuint createtexture(char filtertype);


	//Copies a plane from another .MPC file.
	void getplane(int dstplane,Mpc *source,int srcplane);	//Copies a certain color plane from another MPC object.
	void flipplane(int plane1,int plane2); //Flips two color planes.
	void cut(Mpc *source,int left,int top,int right,int bottom); //This object will contain an image from the subimage of the src object.
	

	void dumpplane(int plane,unsigned char *dumppos);


	void renderblack(); //Fills the image with the black color.
	//Fills the image with the specified color.
	void rendercolor(int r=-1,int g=-1,int b=-1,int a=-1);

	//Creates a new image.
	void create(WORD xsize,WORD ysize,WORD bpp);
	//Creates a new image compatible with the specified image. 
	//You can specify another plane count, but if it's -1, it will be
	//the same as the src image.
	void createcompatible(Mpc *src,int planes=-1);

	//Clears the image memory
	void clear();
	Mpc();
	~Mpc();
};



int Mpc::savebmp(char *filnamn)
{
	if (header.bpp == 4) 
	{
		Mpc mpc;
		mpc.create(header.xsize,header.ysize,header.bpp);
		mpc.getplane(0,this,0);
		mpc.getplane(1,this,1);
		mpc.getplane(1,this,2);		
		return(mpc.savebmp(filnamn));;
	}
	if (header.bpp != 3) return(0);

	SDL_Surface *s;

    Uint32 rmask, gmask, bmask;

#if SDL_BYTEORDER == SDL_BIG_ENDIAN
    rmask = 0xff000000;
    gmask = 0x00ff0000;
    bmask = 0x0000ff00;
#else
    rmask = 0x000000ff;
    gmask = 0x0000ff00;
    bmask = 0x00ff0000;
#endif
	int num;
	
	s = SDL_CreateRGBSurfaceFrom(pixels,header.xsize,header.ysize,24,header.xsize*3,rmask,gmask,bmask,0);
	if (!s) MessageBox(NULL,"Could not save BMP","Mpc Error",MB_OK);
	SDL_SaveBMP(s,filnamn);
	SDL_FreeSurface(s);
return(1);
}

void Mpc::dumpplane(int plane,unsigned char *dumppos)
{
	int num;
	for (num = 0; num < header.xsize*header.ysize; num++)
		dumppos[num] = pixels[num*header.bpp+plane];
}

void Mpc::cut(Mpc *source, int left,int top,int right,int bottom)
{
	clear();
//	createcompatible(source);
	int hor = right - left;
	int ver = bottom - top;
	create(hor,ver,source->header.bpp);
	int xnum,ynum,num;
	for (ynum = 0; ynum < ver; ynum++)
	{
		for (xnum = 0; xnum < hor; xnum++)
		{
			for (num = 0; num < header.bpp; num++)
			{
				pixels[num+xnum*header.bpp+ynum*header.xsize*header.bpp] = source->pixels[num+(xnum+left)*header.bpp+(ynum+top)*source->header.xsize*header.bpp];
			}
		}
	}
	
}


void Mpc::createcompatible(Mpc *src,int planes)
{
	clear();
	if (planes != -1)
		create(src->header.xsize,src->header.ysize,planes);
	else create(src->header.xsize,src->header.ysize,src->header.bpp);
}

void Mpc::flipplane(int plane1,int plane2)
{
	Mpc temp;
	temp.createcompatible(this);
	temp.getplane(0,this,plane1);
	getplane(plane1,this,plane2);
	getplane(plane2,&temp,0);	
}


void Mpc::getplane(int dstplane,Mpc *source,int srcplane)
{
	int xnum,ynum;
	for (ynum = 0; ynum < header.ysize; ynum++)
		for (xnum = 0; xnum < header.xsize; xnum++)
		{
			pixels[xnum*header.bpp + ynum*header.xsize*header.bpp + dstplane] = source->pixels[xnum*source->header.bpp + ynum*source->header.xsize*source->header.bpp + srcplane];
		}
}


void Mpc::rendercolor(int r,int g,int b,int a)
{
	if (!pixels) return;
	long num;
	for(num = 0; num < header.xsize*header.ysize; num++)
	{
		if (r != -1)
		pixels[num*header.bpp] = (unsigned char)r;
		if (g != -1)
		pixels[num*header.bpp+1] = (unsigned char)g;
		if (b != -1)
		pixels[num*header.bpp+2] = (unsigned char)b;
		if (a != -1)
		pixels[num*header.bpp+3] = (unsigned char)a;
	}
}

void Mpc::renderblack()
{
	if (!pixels) return;
	ZeroMemory(pixels,header.xsize*header.ysize*header.bpp);
}



int Mpc::loadbmp4(char *filnamn,char *filnamn2,int plane)
{
	Mpc mpc1;
	Mpc mpc2;
	mpc1.loadbmp(filnamn);
	if (filnamn2)
	mpc2.loadbmp(filnamn2);


	create(mpc1.header.xsize,mpc1.header.ysize,4);
//	int xnum,ynum;

	getplane(0,&mpc1,0);
	getplane(1,&mpc1,1);
	getplane(2,&mpc1,2);
	getplane(3,&mpc2,plane);


	mpc1.clear();
	mpc2.clear();
	return(1);
}



void Mpc::dumpimage(char *file)
{
	ofstream fin;
	fin.open(file,ios::trunc);

	long num;
	int num2;
	for (num = 0; num < header.xsize*header.ysize; num++)
	{
		for (num2 = 0; num2 < header.bpp; num2++)
		{
			fin << (int)pixels[num*header.bpp + num2] << ",";
		}
		fin << "\n";
	}

	fin.close();
}


void Mpc::create(WORD xsize,WORD ysize,WORD bpp)
{
	clear();
	pixels = new unsigned char[xsize*ysize*bpp];
	ZeroMemory(pixels,xsize*ysize*bpp);
	header.xsize = xsize;
	header.ysize = ysize;
	header.bpp = bpp;
}





int Mpc::loadbmpwithmask(char *filnamn,unsigned char r,unsigned char g,unsigned char b)
{
	Mpc mpc;
	if (!mpc.loadbmp(filnamn)) return(0);
	int t = 0;	

	create(mpc.header.xsize,mpc.header.ysize,4);
	int xnum,ynum;
	for (ynum = 0; ynum < header.ysize; ynum++)
		for (xnum = 0; xnum < header.xsize; xnum++)
		{
			pixels[xnum*4 + ynum*header.xsize*4] = mpc.pixels[xnum*3 + ynum*header.xsize*3];
			pixels[xnum*4 + ynum*header.xsize*4+1] = mpc.pixels[xnum*3 + ynum*header.xsize*3+1];
			pixels[xnum*4 + ynum*header.xsize*4+2] = mpc.pixels[xnum*3 + ynum*header.xsize*3+2];

			if (pixels[xnum*4 + ynum*header.xsize*4] == r &&
				pixels[xnum*4 + ynum*header.xsize*4+1] == g &&
				pixels[xnum*4 + ynum*header.xsize*4+2] == b)
			pixels[xnum*4 + ynum*header.xsize*4 + 3] = 0;
			else
				pixels[xnum*4 + ynum*header.xsize*4 + 3] = 255;
		}


	mpc.clear();
	return(1);
}





GLuint Mpc::createtexture(char filtertype)
{



		glGenTextures(1, &texture);	
		GLenum format;
		if (header.bpp == 3) format = GL_RGB;
		else format = GL_RGBA;

	
		if (!filtertype)
		{
		// Create Nearest Filtered Texture
			glBindTexture(GL_TEXTURE_2D, texture);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
			glTexImage2D(GL_TEXTURE_2D, 0, header.bpp, header.xsize, header.ysize, 0, format, GL_UNSIGNED_BYTE, pixels);
		}
		if (filtertype == 1)
		{
		// Create Linear Filtered Texture
			glBindTexture(GL_TEXTURE_2D, texture);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
			glTexImage2D(GL_TEXTURE_2D, 0, header.bpp, header.xsize,header.ysize, 0, format, GL_UNSIGNED_BYTE, pixels);
		}
		if (filtertype == 2)
		{
		// Create MipMapped Texture
			glBindTexture(GL_TEXTURE_2D, texture);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
			gluBuild2DMipmaps(GL_TEXTURE_2D, header.bpp, header.xsize,header.ysize, format, GL_UNSIGNED_BYTE, pixels);
		}
		return(texture);


}


int Mpc::loadbmp(char *filnamn)
{

	clear();

	SDL_Surface *bmp;

	// Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit

	if (bmp = SDL_LoadBMP(filnamn))
	{




		SDL_LockSurface(bmp);

		if (bmp->format->BitsPerPixel != 24)
			MessageBox(NULL,"FEEL!","",MB_OK);


		header.xsize = bmp->w;
		header.ysize = bmp->h;
		header.bpp = 3;
		
		pixels = new unsigned char[header.xsize*header.ysize*header.bpp];
		memcpy(pixels,bmp->pixels,header.xsize*header.ysize*header.bpp);

		SDL_UnlockSurface(bmp);

		SDL_FreeSurface(bmp);

		flipplane(0,2);

		return(1);

	}

	return(0);
	
}


int Mpc::load(char *filnamn)
{
	clear();

	MPC_VERSIONHEADER version;

	ifstream fin;
	fin.open(filnamn,ios::binary);
	
		fin.read((char*)&version,sizeof(version));
		if (version.version != MPC_CURRENTVERSION || fin.fail() || fin.eof())
			return(0);;
		fin.read((char*)&header,sizeof(header));
		pixels = new unsigned char[header.xsize*header.ysize*header.bpp];
		fin.read((char*)pixels,header.xsize*header.ysize*header.bpp);
	fin.close();
	return(1);
}


int Mpc::save(char *filnamn)
{
	MPC_VERSIONHEADER version;
	version.version = MPC_CURRENTVERSION;
	ofstream fin;

	fin.open(filnamn,ios::binary | ios::trunc);
		if (fin.fail()) return(0);
		
		fin.write((char*)&version,sizeof(version));
		fin.write((char*)&header,sizeof(header));
		fin.write((char*)pixels,header.xsize*header.ysize*header.bpp);

	fin.close();
	return(1);
}


void Mpc::clear()
{
	if (pixels) delete[] pixels;
	pixels = NULL;
}

Mpc::Mpc()
{
	pixels = NULL;
	texture = -1;
}

Mpc::~Mpc()
{
	clear();
}








#endif