#ifndef __MYMATH__
#define __MYMATH__

#include <math.h>
#include "mystuff/glvector2f.h"
//Is a set of mathematical routines.



void nurbline(float l0,float l1,float k0,float k1,float *ret,float amount)
{
	float ta,tb,step,x;
	int num,end;

	tb = l1-l0-2.0f*k0/3.0f-k1/3;
	tb *= 6;
	ta = k1-k0-tb;
	ta *= 2;
	end = amount;
	num = 0;
	x = 0;
	step = 1/amount;
	while (num < amount)
	{
		ret[num] = (ta*x*x*x)/6.0f + (tb*x*x)/2.0f + k0*x + l0;

		num++;
		x += step;
	}


}

//NurblineA,B,C does the same thing as NurbLine, but in different ways.
//(The result is almost the same)

void nurblinea(float l0,float l1,float k0,float k1,float *ret,float amount,float ta)
{
	float tb,tc,step,x;
	int num,end;


	tc = l1 + ta/72.0f - k1/3.0f - 2*k0/3.0f - l0;
	tc *= 6.0f;
	tb = k1 - k0 - tc - ta/6.0f;
	tb *= 2;


	end = amount;
	num = 0;
	x = 0;
	step = 1/amount;
	while (num < amount)
	{
		ret[num] = (ta*x*x*x*x)/24.0f + (tb*x*x*x)/6.0f + (tc*x*x)/2 + k0*x + l0;

		num++;
		x += step;
	}


}




void nurblineb(float l0,float l1,float k0,float k1,float *ret,float amount,float tb)
{
	float ta,tc,step,x;
	int num,end;


	ta = k1/2.0f - tb/12.0f + k0/2.0f + l0 - l1;
	ta *= 24;
	tc = k1 - k0 - tb/2.0f - ta/6.0f;

	end = amount;
	num = 0;
	x = 0;
	step = 1/amount;
	while (num < amount)
	{
		ret[num] = (ta*x*x*x*x)/24.0f + (tb*x*x*x)/6.0f + (tc*x*x)/2 + k0*x + l0;

		num++;
		x += step;
	}


}




void nurblinec(float l0,float l1,float k0,float k1,float *ret,float amount,float tc)
{
	float ta,tb,step,x;
	int num,end;

	ta = k1/3.0f + 2*k0/3.0f + tc/6.0f + k0 + l0 - l1;
	ta *= 72;

	tb = k1 - k0 - tc - ta/6.0f;
	tb *= 2;

	end = amount;
	num = 0;
	x = 0;
	step = 1/amount;
	while (num < amount)
	{
		ret[num] = (ta*x*x*x*x)/24.0f + (tb*x*x*x)/6.0f + (tc*x*x)/2 + k0*x + l0;

		num++;
		x += step;
	}


}




#include "mystuff/glvector2f.h"


//Returns true if point is in circle

int isincircle(GLVECTOR2f *point,GLVECTOR2f *circleorigin,float radius)
{
	static GLVECTOR2f v;
	v = *point-*circleorigin;
	if (sqrt(v.x*v.x+v.y*v.y) <= radius) return(1); else return(0);

}

//Returns the position of the line (0 - 1) that crossed the circle, or -1 if it didn't
float crossingcircle(GLVECTOR2f *vertex1,GLVECTOR2f *vertex2,GLVECTOR2f *point,float radius,float *anotherret)
{
	float ret1,ret2;
	GLVECTOR2f v;
	GLVECTOR2f vd;
	float p,q;

	v = *vertex1-*point;
	vd = *vertex2 - *vertex1;


	p = v.x*vd.x+v.y*vd.y;
	p /= vd.x*vd.x+vd.y*vd.y;
	q = v.x*v.x+v.y*v.y-radius*radius;
	q /= vd.x*vd.x+vd.y*vd.y;


	ret1 = -p + sqrt(p*p-q);
	ret2 = -p - sqrt(p*p-q);
	if (ret1 < ret2)
	{
		p = ret2;
		ret2 = ret1;
		ret1 = p;
	}

	if (ret1 < 0 && ret1 > 1 && ret2 < 0 && ret2 > 1) return(-1);
	if (ret1 >= 0 && ret1 <= 1 && ret2 >= 0 && ret2 <= 1) 
	{
		if (anotherret)
		*anotherret = ret2;
		return(ret1);
	}
	if (anotherret)
		*anotherret = -1;
	if (ret1 >= 0 && ret1 <= 1)
	{
		return(ret1);
	}
	else return(ret2);
};


//Returns 1 if the point is inside the triangle
int isintriangle(GLVECTOR2f *v1, GLVECTOR2f *v2,GLVECTOR2f *v3,GLVECTOR2f *point)
{
	static GLVECTOR2f dir1;
	static GLVECTOR2f dir2;
	dir1 = *v2 - *v1;
	dir2 = *v3 - *v1;
	static float t,u;

	u = dir1.x*(point->y-v1->y) - dir1.y*(point->x-v1->x);
	u /= dir1.x*dir2.y - dir2.x*dir1.y;

	if (dir1.x)
	{
		t = point->x-v1->x-u*dir2.x;
		t /= dir1.x;
	}
	else
	{
		t = point->y-v1->y-u*dir2.y;
		t /= dir1.y;
	}
	if (t >= 0 && u >= 0 && t + u <= 1) 
		return(1);
	return(0);
}




float isinline(GLVECTOR2f *v1, GLVECTOR2f *v2,GLVECTOR2f *point)
{
	float diff;
	diff = (point->x - v1->x) / (v2->x - v1->x);
	diff = v1->y + diff*(v2->y-v1->y);
	diff -= point->y;
	if (diff < 0.000001 && diff > -0.000001) return(1);
	return(0);	
}


//Returns the percentage of the line (0 - 1) that crossed the other line, or
//-1 if they didn't.
float crossingline(GLVECTOR2f *vertex11,GLVECTOR2f *vertex12,GLVECTOR2f *vertex21,GLVECTOR2f *vertex22,float *line2len = NULL)
{
	static GLVECTOR2f dir1;
	static GLVECTOR2f dir2;
	dir1 = *vertex12 - *vertex11;
	dir2 = *vertex22 - *vertex21;

	static float len;
	len = dir2.x*(vertex21->y-vertex11->y)-dir2.y*(vertex21->x-vertex11->x);
	len /= dir1.y*dir2.x - dir1.x*dir2.y;

	static float len2;
	if (dir2.x)
	{
	len2 = len*dir1.x-vertex21->x+vertex11->x;
	len2 /= dir2.x;
	}
	else 
	{
		len2 = len*dir1.y-vertex21->y+vertex11->y;
		len2 /= dir2.y;
	}
	

	if (len > 1 || len < 0 || len2 < 0 || len2 > 1) 
		return(-1);
	if (line2len) *line2len = len2;
	return(len);
}






GLVECTOR3f calcnormal(GLVECTOR3f d1,GLVECTOR3f d2)
{
	GLVECTOR3f temp;	
	temp.x = d1.y*d2.z - d2.y*d1.z;
	temp.y = d1.z*d2.x - d2.z*d1.x;
	temp.z = d1.x*d2.y - d2.x*d1.y;
	return(temp);
}







#endif