#ifndef __PLANE__
#define __PLANE__

#include "mystuff/glvector2f.h"
#include "mystuff/glvector3f.h"
#include "mystuff/debug.h"


//A plane handler. Sets up a grid easy to render. Normals are loaded BUT UNTOUCHED. 
//texcoord is loaded and set as 0,0 at top left and 1,1 at bottom right.

template <class VECTOR> 
class Plane
{
public:
	VECTOR *vertex;
	GLVECTOR2f *texcoord;
	GLVECTOR3f *normal;
	int normals;
	int *vertice;
	int vertexes;
	int xlen,ylen;
	int amx,amy; //xlen and amx is actually the same....
	float width,height;
	int vertices;	
	//amx and amy tells how many VERTICES in a row/column. The plane physical size is (amx-1)*step
	void creategrid(int _amx,int _amy,float step);
	void copygrid(Plane<VECTOR> *src);

	void clear();
	void dump(char *fil);	
	Plane();
	~Plane();
};



template <class VECTOR> 
void Plane<VECTOR>::copygrid(Plane<VECTOR> *src)
{
	clear();
	vertex = new VECTOR[src->vertexes];
	normal = new GLVECTOR3f[src->normals];
	texcoord = new GLVECTOR2f[src->vertexes];
	vertice = new int[src->vertices];
	vertexes = src->vertexes;
	normals = src->normals;
	xlen = src->xlen;
	ylen = src->ylen;
	width = src->width;
	height = src->height;
	vertices = src->vertices;
	amx = src->amx;
	amy = src->amy;

	memcpy(vertex,src->vertex,sizeof(VECTOR)*vertexes);
	memcpy(texcoord,src->texcoord,sizeof(GLVECTOR2f)*vertexes);
	memcpy(vertice,src->vertice,sizeof(int)*vertices);
	memcpy(normal,src->normal,sizeof(GLVECTOR3f)*normals);
}



template <class VECTOR> 
Plane<VECTOR>::Plane()
{
	vertex = NULL;
	vertice = NULL;
	texcoord = NULL;
	normal = NULL;
	normals = 0;
	vertexes = 0;
	vertices = 0;

}

template <class VECTOR> 
void Plane<VECTOR>::clear()
{
	if (vertex) 
	{
		delete[] vertex;
		vertex = NULL;
	}
	if (texcoord) 
	{ 
		delete[] texcoord;
		texcoord = NULL;
	}
	if (vertice) 
	{
		delete[] vertice;
		vertice = NULL;
	}
	if (normal)
	{
		delete[] normal;
		normal = NULL;
	}
}

template <class VECTOR>
Plane<VECTOR>::~Plane()
{
	clear();
}




template <class VECTOR> 
void Plane<VECTOR>::creategrid(int _amx,int _amy,float step)
{
	amx = _amx;
	amy = _amy;

	clear();

	int xnum,ynum;
	xlen = amx;
	ylen = amy;
	width = (float)(amx-1)*step;
	height = (float)(amy-1)*step;
	
	//amount vertexes is amx*amy
	vertex = new VECTOR[amx*amy];
	normal = new GLVECTOR3f[amx*amy];
	texcoord = new GLVECTOR2f[amx*amy];
//	vertice = new int[amx*amy*2-amy*2-amx+2];
	vertice = new int[amy*2*(amx-1)];

	
	
	for (ynum = 0; ynum < amy; ynum++)
		for (xnum = 0; xnum < amx; xnum++)
		{
			vertex[xnum+ynum*xlen].set((float)xnum*step,(float)ynum*step);
			texcoord[xnum+ynum*xlen].set(((float)xnum*step)/width,((float)ynum*step)/height);
		}

	xnum = 0;
	ynum = 0;
	int vindex = 0;
	while(1)
	{

		if (xnum >= xlen - 1) break;
		while(ynum < ylen)
		{
			vertice[vindex] = xnum+ynum*xlen;
			vindex++;
			vertice[vindex] = xnum+ynum*xlen + 1;
			vindex++;
			ynum++;
		}
		ynum--;
		xnum++;
		if (xnum >= xlen - 1) break;
		while (ynum >= 0)
		{
			vertice[vindex] = xnum+ynum*xlen;
			vindex++;
			vertice[vindex] = xnum+ynum*xlen + 1;
			vindex++;
			ynum--;
		}


/*		while(ynum >= 0)
		{
			if (ynum < ylen-1)
			{
				vertice[vindex] = xnum+ynum*xlen;
				vindex++;
			}

			if (ynum > 0)
			{
				vertice[vindex] = xnum+ynum*xlen + 1;
				vindex++;
			}
			ynum--;
		}
*/		
		
		ynum++;
		xnum++;
	}
	vertices = vindex;
	vertexes = normals = amx*amy;
}



template <class VECTOR> 
void Plane<VECTOR>::dump(char *fil)
{
	DebugClass debug;

	debug.open(fil,0);
	debug.put("PLANE DUMP:\n\nVertexes: ");
	debug.put(xlen*ylen);
	debug.newline();
	int num;
	for (num = 0; num < xlen*ylen; num++)
	{
		debug.put(num);
		debug.put(": ");
		debug.put(vertex[num].x);
		debug.put(", ");
		debug.put(vertex[num].y);
		debug.put(" | ");
		debug.put(texcoord[num].x);
		debug.put(", ");
		debug.put(texcoord[num].y);
		debug.newline();
	}
	


	debug.newline();
	debug.put("Vertices: ");
	debug.put(vertices);
	debug.newline();

	for (num = 0; num < vertices; num++)
	{
		debug.put(num);
		debug.put(": ");
		debug.put(vertice[num]);
		debug.newline();
	}

	

	debug.close();
}


#endif