#ifndef __TEXTUREPAINTER__
#define __TEXTUREPAINTER__



struct TPVertex
{
	GLuint texture[2];
	float texturestrength[2];
	void addtexture(GLuint tex,float strength,int index);
	TPVertex();
	int getindex(GLuint tex);
	int getweakestindex();
	float calcalpha(GLuint basetex,int baseindex);
	void flip();
};


void TPVertex::flip()
{
	float strtemp;
	GLuint textemp;
	textemp = texture[0];
	strtemp = texturestrength[0];
	texture[0] = texture[1];
	texturestrength[0] = texturestrength[1];
	texture[1] = textemp;
	texturestrength[1] = strtemp;
}

float TPVertex::calcalpha(GLuint basetex,int baseindex)
{
	if (baseindex != getindex(basetex)) return(1-baseindex);
	float sum;
	if (texture[0] == -1 && texture[1] == -1)
	{
		return(0);
	}
	else if (texture[0] == -1)
	{
		return(1);
	}
	else if (texture[1] == -1)
	{
		return(0);
	}
	else
	{
		sum = pow(texturestrength[0],2)+pow(texturestrength[1],2);
		if (sum == 0) return(0.5); //Both strengthes are 0. //Should not happen.
		return(pow(texturestrength[1],2)/sum);
	}
}



TPVertex::TPVertex()
{
	texture[0] = texture[1] = -1;
	texturestrength[0] = texturestrength[1] = -1;
}


int TPVertex::getweakestindex()
{
	if (texturestrength[0] <= texturestrength[1]) return(0); else return(1);
}




int TPVertex::getindex(GLuint tex)
{
	if (texture[0] == tex) return(0);
	else if (texture[1] == tex) return(1);
	return(-1);

}





void TPVertex::addtexture(GLuint tex, float strength,int index)
{
//	int index;
//	index = getindex(tex);
//	if (index == -1) index = getweakestindex();
	texture[index] = tex;
	texturestrength[index] = strength;
}







class TexturePainter
{
public:
	World *target;
	TPVertex *vertex;
	int vertexes; // vertexes = amx*amy;
	int amx,amy;

	void settarget(World *t);
	void addtexture(char *bmpname, char *texturename,int plane);

	void write();
	void smooth();


	void fliptofit();

	int oktoaddtexture(int x,int y,int plane,GLuint tex,float str);


	void clear();
	TexturePainter();
	~TexturePainter();


};


int TexturePainter::oktoaddtexture(int x,int y,int plane,GLuint tex,float str)
{
	if (x == 0 && y == 0) return(1);
	
	if (vertex[x+y*amx].texture[plane] != -1 && vertex[x+y*amx].texture[plane] != tex && vertex[x+y*amx].texturestrength[plane] > 0) return(0);
	if (vertex[x+y*amx+1].texture[plane] != -1 && vertex[x+y*amx+1].texture[plane] != tex && vertex[x+y*amx+1].texturestrength[plane] > 0) return(0);
	if (vertex[x+(y+1)*amx].texture[plane] != -1 && vertex[x+(y+1)*amx].texture[plane] != tex && vertex[x+(y+1)*amx].texturestrength[plane] > 0) return(0);

	if (x > 0)
	{
		if (vertex[x+y*amx-1].texture[plane] != -1 && vertex[x-1+y*amx].texture[plane] != tex && vertex[x+y*amx].texturestrength[plane] > 0) return(0);

	}
}







void TexturePainter::write()
{
	int xnum,ynum,xnum2,ynum2;
	int facepitch = (target->sector[0].amx-1)*2;
	int famx = target->sector[0].amx-1;
	int famy = target->sector[0].amy-1;
	Sector *s;
	TPVertex *v;

	SectorFace *f,*f2;
	


	for (ynum = 0; ynum < target->amy; ynum++)
		for (xnum = 0; xnum < target->amx; xnum++)
		{
			s = &target->sector[xnum+ynum*target->amx];
			famx = s->amx-1;
			famy = s->amy-1;
			facepitch = famx*2;
			for (ynum2 = 0; ynum2 < famy; ynum2++)
				for (xnum2 = 0; xnum2 < famx; xnum2++)
				{
					v = &vertex[xnum*famx+xnum2+(ynum*famy+ynum2)*amx];
					f = &s->face[xnum2*2+ynum2*facepitch];
					f2 = &s->face[1+xnum2*2+ynum2*facepitch];
					f->texture = v->texture[0];
					f->texture2 = v->texture[1];
					f2->texture = v->texture[0];
					f2->texture2 = v->texture[1];
				
				}
		}

	int index1,index2;
	float val;
	GLuint bigtex;


	for (ynum = 0; ynum < target->amy; ynum++)
		for (xnum = 0; xnum < target->amx; xnum++)
		{
			s = &target->sector[xnum+ynum*target->amx];
			famx = s->amx-1;
			famy = s->amy-1;
			facepitch = famx*2;
			for (ynum2 = 0; ynum2 < famy; ynum2++)
				for (xnum2 = 0; xnum2 < famx; xnum2++)
				{
					v = &vertex[xnum*famx+xnum2+(ynum*famy+ynum2)*amx];
					f = &s->face[xnum2*2+ynum2*facepitch];
					f2 = &s->face[1+xnum2*2+ynum2*facepitch];

					bigtex = v->texture[1];
					f->color2[0].a = v->calcalpha(bigtex,1);				
					

					v = &vertex[xnum*famx+1+xnum2+(ynum*famy+ynum2)*amx];


					val = v->calcalpha(bigtex,1);
					f->color2[1].a = val;	
					f2->color2[1].a = val;	


					v = &vertex[xnum*famx+xnum2+(ynum*famy+ynum2+1)*amx];

					val = v->calcalpha(bigtex,1);
					f->color2[2].a = val;	
					f2->color2[2].a = val;	


					v = &vertex[xnum*famx+xnum2+1+(ynum*famy+ynum2+1)*amx];

					val = v->calcalpha(bigtex,1);
					f2->color2[0].a = val;	

				}
		}



}



void TexturePainter::addtexture(char *bmpname, char *texturename,int plane)
{
	Mpc mpc;
	mpc.loadbmp(bmpname);
	mpc.savebmp("data\\dumps\\facit.bmp");
	GLuint tex = currentworld->texturehandler.gettexture(currentworld->texturehandler.loadbmp(texturename));
	int xnum,ynum,index;
	for (ynum = 0; ynum < amy-1; ynum++)
		for (xnum = 0; xnum < amx-1; xnum++)
		{
			if (mpc.pixels[xnum*mpc.header.bpp+ynum*mpc.header.xsize*mpc.header.bpp])
			{
				vertex[xnum+ynum*amx].addtexture(tex,(float)mpc.pixels[xnum*mpc.header.bpp+ynum*mpc.header.xsize*mpc.header.bpp]/255.0f,plane);
			}
		}

}



void TexturePainter::settarget(World *t)
{
	clear();
	target = t;
	amx = t->amx*(t->sector[0].amx-1)+1;
	amy = t->amy*(t->sector[0].amy-1)+1;
	vertexes = amx*amy;
	vertex = new TPVertex[vertexes];
}



void TexturePainter::clear()
{
	if (vertex) 
	{
		delete[] vertex;
		vertex = NULL;
	}
}


TexturePainter::TexturePainter()
{
	target = NULL;
	vertex = NULL;
}


TexturePainter::~TexturePainter()
{
	clear();
}



#endif