#ifndef __ENTITY_WATER__
#define __ENTITY_WATER__


#include "entity.h"
#include "mystuff/plane.h"

#define ENTITY_TYPE_WATER 1
#define ENTITY_TYPE_UNDERWATERFADER 3


struct EN_Water_Save
{
	GLVECTOR2f texcoord[4];
	float xlen,ylen;
};


class EN_Water : public Entity
{
public:
	GLVECTOR3f corner[4];
	GLVECTOR2f texcoord[4];

	Plane<GLVECTOR3f> plane;
	Plane<GLVECTOR3f> oplane;
	float *vertexdir;
	float dirspeed;
	float dirlen;

	float xlen,ylen;


	GLuint texture;

	int init();
	int moveme();
	void rendercycle();

	void set(float left,float top,float right,float bottom,float height);
	void settex(float left,float top,float texlen);
	void settexture(GLuint tex) {texture = tex;};


	void setupplane();


	int afterload();
	int beforesave();


	int renderme();
	EN_Water();
	void clear();
	~EN_Water();


};


int EN_Water::afterload()
{
	EN_Water_Save *h = (EN_Water_Save*)saveinformation;
	texcoord[0] = h->texcoord[0];
	texcoord[1] = h->texcoord[1];
	texcoord[2] = h->texcoord[2];
	texcoord[3] = h->texcoord[3];
	xlen = h->xlen*3.0f;
	ylen = h->ylen*3.0f;
	delete[] saveinformation;
	saveinformation = 0;
	setupplane();
	return(1);
}


int EN_Water::beforesave()
{
	EN_Water_Save *h = new EN_Water_Save;
	h->texcoord[0] = texcoord[0];
	h->texcoord[1] = texcoord[1];
	h->texcoord[2] = texcoord[2];
	h->texcoord[3] = texcoord[3];
	h->xlen = xlen;
	h->ylen = ylen;

	saveinformation = h;
	saveinformationsize = sizeof(EN_Water_Save);
	
	return(1);

}



void EN_Water::setupplane()
{
	GLVECTOR3f v;
	float len;

	int xnum,ynum;
	for (xnum = 0; xnum < plane.vertexes; xnum++)
		plane.vertex[xnum] *= GLVECTOR3f(xlen/3.0f,1,ylen/3.0f);


	for (ynum = 0; ynum < plane.amy; ynum++)
		for (xnum = 0; xnum < plane.amx; xnum++)
		{
			v = plane.vertex[xnum+ynum*plane.amx] + pos;

			v.y = 0;
			len = v.len();
			vertexdir[xnum+ynum*plane.amx] = len*10;
		}

	dirspeed = 0.01;
	dirlen = 0.9;

	
	
	oplane.copygrid(&plane);



}

EN_Water::~EN_Water()
{
	clear();
}

void EN_Water::clear()
{
	if (vertexdir)
	{
		delete[] vertexdir;
		vertexdir = NULL;
	}
}


int EN_Water::init()
{
	plane.creategrid(4,4,1);
	int num,xnum,ynum;
	clear();
	vertexdir = new float[plane.vertexes];
	for (num = 0; num < plane.vertexes; num++)
	{
		plane.vertex[num].z = plane.vertex[num].y;
		plane.vertex[num].y = 0;	
	}

	setupplane();

	texture = currentworld->texturehandler.gettexture(currentworld->texturehandler.load("data\\textures\\water.mpc",2));


	
	return(1);
}



int EN_Water::moveme()
{

	return(1);
}


void EN_Water::rendercycle()
{
	static int num;
	for (num = 0; num < plane.vertexes; num++)
	{
		plane.vertex[num] = oplane.vertex[num];
		plane.texcoord[num] = oplane.texcoord[num];
		plane.texcoord[num].x += cos(currentworld->worldTickCount*dirspeed*0.04)*0.05;
		plane.texcoord[num].y += sin(currentworld->worldTickCount*dirspeed*0.04)*0.05;
//		plane.vertex[num].x += cos(vertexdir[num])*dirlen;
//		plane.vertex[num].z += sin(vertexdir[num])*dirlen;
		plane.vertex[num].y += sin(vertexdir[num]+currentworld->worldTickCount*dirspeed*0.1)*dirlen;
//		vertexdir[num] += dirspeed;
	}

}


EN_Water::EN_Water()
{
	type = ENTITY_TYPE_WATER;
	vertexdir = NULL;
	xlen = ylen = 1;
}

int EN_Water::renderme()
{

	if (homesector == ENTITY_HOMELESS) return(0); //Must have a home sector.

	if (currentworld->sector[homesector].getminheight() > currentworld->waterlevel) return(1);

	rendercycle();
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glBindTexture(GL_TEXTURE_2D,texture);
	glColor4f(1,1,1,0.4);

	
	
	static int num;
	glBegin(GL_TRIANGLE_STRIP);
		for (num = 0; num < plane.vertices; num++)
		{
			glTexCoord2fv((float*)&plane.texcoord[plane.vertice[num]]);
			glVertex3fv((float*)&plane.vertex[plane.vertice[num]]);
		}
	glEnd();

	glDisable(GL_BLEND);
	return(1);
}

void EN_Water::settex(float left,float top,float texlen)
{
	float xlen = (corner[1].x - corner[0].x) / texlen;
	float ylen = (corner[3].z - corner[0].z) / texlen;
	texcoord[0].set(left,top);
	texcoord[1].set(left+xlen,top);
	texcoord[2].set(left+xlen,top+ylen);
	texcoord[3].set(left,top+ylen);
}


void EN_Water::set(float left,float top,float right,float bottom,float height)
{
	xlen = right-left;
	ylen = bottom-top;
	corner[0].set(left,height,top);
	corner[1].set(right,height,top);
	corner[2].set(right,height,bottom);
	corner[3].set(left,height,bottom);
}




class EN_UnderWaterFader : public Entity
{
public:
	int afterload() {return(1);};
	int beforesave() {return(1);};
	int renderme();
	int move() {return(1);};

	EN_UnderWaterFader() {type = ENTITY_TYPE_UNDERWATERFADER; homesector = ENTITY_STATIC;}
};



int EN_UnderWaterFader::renderme()
{

	if (currentworld->camerapos->y < currentworld->waterlevel)
	{
		glPushMatrix();
		glLoadIdentity();
		glTranslatef(0,0,-1);
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_DEPTH_TEST);
		glColor4f(currentworld->watercolor.x,currentworld->watercolor.y,currentworld->watercolor.z,0.5);
		glBegin(GL_QUADS);
			glVertex3f(-2,1,0);
			glVertex3f(2,1,0);
			glVertex3f(2,-1,0);
			glVertex3f(-2,-1,0);
		glEnd();
		glDisable(GL_BLEND);
		glEnable(GL_DEPTH_TEST);
		glEnable(GL_TEXTURE_2D);
		glPopMatrix();
	}
	return(1);
}



#endif