#ifndef __WORLD__
#define __WORLD__


#include "sector.h"

#define WORLDVERSION 100


#define WORLD_MAX_ENTITIES 32000

struct WorldFileHeader
{
	WORD version;
	WORD amx,amy;
	float xlen,ylen;

	/* Gamplay attributes */

	float save_viewdistance; 
	float save_viewdifflen; 
	
	float waterlevel; 
	GLVECTOR3f fogcolor; 
	GLVECTOR3f sundir; 


};






class World
{
	

public:
	
	TextureHandler texturehandler;

	GLVECTOR3f *camerapos;
	GLVECTOR3f *cameradir;


	Sector *sector;
	int sectors;

	int *renderlist; //Sectors defines the size of this field
	int renderlistcount; //it's NOT the size of rendelist!! Only how many sectors that is currently registered in it.

	int amx,amy; //Tells the dimensions of the sector grid. (sectors = amx*amy)
	float xlen,ylen; //Tells the physical dimension on each sector.



	void create(int worldxsize,int worldysize,int sectorlen,float xstep = 1,float ystep = 1);
	void finalize();
	void render(GLenum mode);
	void setviewdist(float dist,float difflen = -1);
	void setcameraref(GLVECTOR3f *pos) {camerapos = pos;};
	void setcameradirref(GLVECTOR3f *dir) {cameradir = dir;};

	float getheight(float x,float y);

	void makebumpmap(char *filename);


	void v2a(GLVECTOR3f *src,GLVECTOR3f *dst = NULL); // Vector 2 angle
	void a2v(GLVECTOR3f *src,GLVECTOR3f *dst = NULL); // Angle 2 vector


	//DO NOT USE (yet)!!! SLOOOOOW!! (but working)
	float sendray(const GLVECTOR3f *pos,const GLVECTOR3f *dir); //Returns the multiple of dir until it hits the surface, or -1 if it don't


	void setfaceindex();
	void calculatenormals();

	//Manipulative functions:
	void setglobaltexture(GLuint texture);
	void setglobaltexture2(GLuint texture);
	void setglobalcolor(GLVECTOR3f color);
	void setglobalcolor2(GLVECTOR4f color);
	void nudgeedges(float len); //Lowers the boundary vertexes to the specified value
	void modsectorvertexes(GLVECTOR3f v);
	void setbump(float *map,int xsize,int ysize);
	void normalizetexcoords(); //Makes the texcoord have the same coordinate as the vertex

	void setfogcolor(GLVECTOR3f col);

	World *clone(World *target = NULL);


	void testrender(GLenum mode);
	void rendernormals();
	void renderborders();
	void makerenderlist();		


		
	void save(char *file);
	void load(char *file);


	// GAMEPLAY PROPERTIES
	float viewdistance;
	float save_viewdistance; // Saved in .map file, Is viewdistance, without sectorlen modification.
	float save_viewdifflen; // saved in .map file

	
	float waterlevel; //saved in .mapfile
	GLVECTOR3f fogcolor; //saved in .mapfile
	GLVECTOR3f sundir; //saved in .mapfile

	GLVECTOR3f watercolor; //Should be a static entity!
	
	unsigned long worldTickCount; //gets updated every time render is called.


	//ENTITY FUNCTIONS. Manages physical removal after done.
	Entity *entity[WORLD_MAX_ENTITIES];
	int entities;

	Entity *createentitybytype(int type); // Returns an entity with the specified type. */
	int saveentities(char *file);
	int loadentities(char *file);


	void clearentities(); //Physically removes the entities from memory and resets the entity count.

	int addentity(Entity *src); //Add the entity into the list. Returns the entitie's index.
	void deleteentity(Entity *src); //Removes the entity from the list.
	void deleteentity(int index); //Removes the entity in the specified index from the list.
	void deleteentitiesbytype(int type); //Removes all entitites by a certain type.


	void collectentities(); //Collects entities made by entities. (The bringup list)
	void registerentities();
	void loadentitytextures(int index);
	void loadentitytextures(Entity *e);
	void loadallentitytextures();


	void clear();
	World();
	~World();
};
World *currentworld; // This is the point to where all entities will look for binding to world.




void World::v2a(GLVECTOR3f *src, GLVECTOR3f *dst)
{
	static GLVECTOR3f temp;
	if (!dst) dst = &temp;
	dst->x = asin(-src->y)*57.8934f;
	dst->y = 180 + atan2(-src->x,src->z)*57.8934;
	dst->z = 0;
	if (dst == &temp)
	{
		*src = temp;
	}
}

void World::a2v(GLVECTOR3f *src, GLVECTOR3f *dst)
{
	if (!src) 
		return;
	static GLVECTOR3f temp;
	if (!dst) dst = &temp;

	dst->x = sin(0.0174532*src->y)*cos(0.0174532*src->x);
	dst->z = -cos(0.0174532*src->y)*cos(0.0174532*src->x);
	dst->y = -sin(0.0174532*src->x);
	*dst *= -1;

	if (dst == &temp)
	{
		*src = temp;
	}
}

int World::loadentities(char *file)
{
	clearentities();
	ifstream fin;
	fin.open(file,ios::binary);

	int amount;
	int type;
	Entity *e;

	fin.read((char*)&amount,sizeof(amount));
	int num;
	for (num = 0; num < amount; num++)
	{
		fin.read((char*)&type,sizeof(int));
		e = createentitybytype(type);
		if (!e) 
		{
			MessageBox(NULL,"Invalid type when loading entities!","Error",MB_OK);
			return(0);
		}
		e->load(&fin);
		loadentitytextures(addentity(e));
		e->afterload();
	}


	registerentities();

	fin.close();
	return(1);
}


int World::saveentities(char *file)
{
	collectentities();

	
	ofstream fout;
	fout.open(file,ios::binary | ios::trunc);
	fout.write((char*)&entities,sizeof(entities));
	int num;

	for (num = 0; num < entities; num++)
	{
		fout.write((char*)&entity[num]->type,sizeof(int));
		entity[num]->save(&fout);
	}


	fout.close();
	return(1);
}


void World::setfogcolor(GLVECTOR3f col)
{
	fogcolor = col;
	glFogfv(GL_FOG_COLOR,(float*)&fogcolor);
	glClearColor(fogcolor.x,fogcolor.y,fogcolor.z,1);
}


void World::makebumpmap(char *filename)
{
	Mpc mpc;
	int sax,say;
	sax = (sector[0].amx-1);
	say = (sector[0].amy-1);
	float *buf;
	mpc.create(sax*amx+1,say*amy+1,3);
	buf = new float[(sax*amx+1)*(say*amy+1)];

	float highest,lowest,ret;
	highest = 0;
	lowest = 999;

	int xnum,ynum,xnum2,ynum2;
	for (ynum = 0; ynum < amy; ynum++)
		for (xnum = 0; xnum < amx; xnum++)
			for (ynum2 = 0; ynum2 < sector[xnum+ynum*amx].amy; ynum2++)
				for (xnum2 = 0; xnum2 < sector[xnum+ynum*amx].amx; xnum2++)
				{
					ret = sector[xnum+ynum*amx].vertex[xnum2+ynum2*sector[xnum+ynum*amx].amx].y;
					if (ret >= waterlevel)
					buf[xnum*sax+xnum2+(ynum*say+ynum2)*(amx*sax+1)] = ret;
					else buf[xnum*sax+xnum2+(ynum*say+ynum2)*(amx*sax+1)] = -1;
					if (ret < lowest) lowest = ret;
					if (ret > highest) highest = ret;

				}

	highest -= lowest;	

	for (xnum = 0; xnum < (sax*amx+1)*(say*amy+1); xnum++)
	{
		
		buf[xnum] -= lowest;
		buf[xnum] /= highest;
		if (buf[xnum] < 0) buf[xnum] = 0;
		mpc.pixels[xnum*3] = (unsigned char)(255.0f*buf[xnum]);
	}


	mpc.getplane(1,&mpc,0);
	mpc.getplane(2,&mpc,0);
	mpc.savebmp(filename);
	delete[] buf;
}






void World::collectentities()
{
	static int num,num2;
	for (num = 0; num < entities; num++)
	{
		for (num2 = 0; num2 < entity[num]->ambringup; num2++)
		{
			if (entity[num]->bringup[num2]->parent) // if it has a parent, it only needs to load it's textures
			{
				loadentitytextures(entity[num]->bringup[num2]);
			}
			else
			{
				addentity(entity[num]->bringup[num2]);
			}
		}
		entity[num]->ambringup = 0;
		if (entity[num]->REMOVEME)
			deleteentity(num);
	}
}

void World::loadallentitytextures()
{
	int num;
	for (num = 0; num < entities; num++)
		loadentitytextures(num);
}

void World::loadentitytextures(int index)
{
	loadentitytextures(entity[index]);
}

void World::loadentitytextures(Entity *e)
{
	M3o *m;
	int anim,num,mat;
	for (anim = 0; anim < ENTITY_MAXANIMATIONS; anim++)
	{
		if (!e->model[anim]) continue;

		
		for (num = 0; num < e->model[anim]->animations; num++)
		{
			m = &e->model[anim]->model[num];
			for (mat = 0; mat < m->materials; mat++)
			{
				if (!m->material[mat].texturename[0]) continue;
//				MessageBox(NULL,(char*)m->material[mat].texturename,"",MB_OK);
				m->material[mat].texture = texturehandler.gettexture(texturehandler.load((char*)m->material[mat].texturename));								
			}
		}
	}
}




void World::registerentities()
{
	int num;
	int iindex;

	for (num = 0; num < entities; num++)
	{
		if (entity[num]->homesector < ENTITY_HOMELESS) continue; 
		iindex = (int)(entity[num]->pos.x / xlen) + (int)(entity[num]->pos.z / ylen)*amx;
		if (entity[num]->homesector != iindex)
		{
			if (entity[num]->homesector != ENTITY_HOMELESS) 
				sector[entity[num]->homesector].deleteentity(entity[num]);
			sector[iindex].addentity(entity[num]);
			entity[num]->homesector = iindex;
		}
	}

}

void World::deleteentitiesbytype(int type)
{
	static int num;
	for (num = 0; num < entities; num++)
	{
		if (entity[num]->type == type) { deleteentity(num); num--;}
	}
}


void World::deleteentity(Entity *src)
{
	static int num,num2;
	for (num = 0; num < entities; num++)
	{
		if (entity[num] == src) 
			deleteentity(num);
	}
}


void World::deleteentity(int index)
{
	static int num;
	if (entity[index]->homesector > ENTITY_HOMELESS) sector[entity[index]->homesector].deleteentity(entity[index]);
	delete entity[index];
	entities--;
	for (num = index; num < entities; num++)
	{
		entity[num] = entity[num+1];
	}
}


int World::addentity(Entity *src)
{
	entity[entities] = src;
	entities++;
	src->init();
	loadentitytextures(entities-1);
	return(entities-1);
}

void World::clearentities()
{

	collectentities();
	registerentities();

	while (entities)
		deleteentity(0);
	entities = 0;
}


World *World::clone(World *target)
{
	if (!target)
		target = new World;

	memcpy(target,this,sizeof(World));

	target->renderlist = new int[sectors];
	memcpy(target->renderlist,renderlist,sizeof(int)*sectors);

	int num;
	if (sectors)
	{
		target->sector = new Sector[sectors];
		if (!target->sector)
		{
			MessageBox(NULL,"Out of memory","",MB_OK);
			exit(0);
		}
	
		for (num = 0; num < sectors; num++)
			sector[num].clone(&target->sector[num]);

		texturehandler.clone(&target->texturehandler);
	}
	else target->sector = NULL;

	return(target);

}


float World::sendray(const GLVECTOR3f *pos,const GLVECTOR3f *dir)
{
	static int index;
	static float ret;
	static float r;
	static int indice[1024];
	static int indices;
	static int num;
	ret = -1;
	



	for (index = 0; index < sectors; index++)
	{
		sector[index].getcrossingfaces(*pos,*dir,indice,1024,&indices);
		for (num = 0; num < indices; num++)
		{
				r = sector[index].intersecting(indice[num],*pos,*dir);
				if (r != -1 && (ret == -1 || r < ret)) ret = r;
		}
	}
	return(ret);
}



void World::save(char *file)
{
	ofstream fout;
	int num;
	WorldFileHeader header;

	header.version = WORLDVERSION;
	header.amx = amx;
	header.amy = amy;
	header.xlen = xlen;
	header.ylen = ylen;
	header.save_viewdistance = save_viewdistance;
	header.save_viewdifflen = save_viewdifflen;
	header.waterlevel = waterlevel;
	header.fogcolor = fogcolor;
	header.sundir = sundir;

	fout.open(file,ios::trunc | ios::binary);

	fout.write((char*)&header,sizeof(header));


	texturehandler.prepareforsave();
	for (num = 0; num < sectors; num++)
		sector[num].reporttextures(&texturehandler);
	texturehandler.finalizeforsave();
	texturehandler.savetextures(&fout);


	for (num = 0; num < sectors; num++) 
		sector[num].save(&fout,&texturehandler);



	fout.close();

}

void World::load(char *file)
{
	clear();
	ifstream fin;
	int num;
	WorldFileHeader header;

	fin.open(file,ios::binary);
	fin.read((char*)&header,sizeof(header));
	if (header.version != WORLDVERSION) return;
	amx = header.amx;
	amy = header.amy;
	xlen = header.xlen;
	ylen = header.ylen;
	sectors = amx*amy;
	save_viewdistance = header.save_viewdistance;
	save_viewdifflen = header.save_viewdifflen;
	waterlevel = header.waterlevel;
	fogcolor = header.fogcolor;
	sundir = header.sundir;


	sector = new Sector[sectors];
	renderlist = new int[sectors];


	texturehandler.loadtextures(&fin);

	for (num = 0; num < sectors; num++)
		sector[num].load(&fin,&texturehandler);

	fin.close();

	setfogcolor(fogcolor);
	setviewdist(save_viewdistance,save_viewdifflen);

}


void World::nudgeedges(float len)
{
	int index,num;
	Sector *s;
	//Upper border
	for (index = 0 ; index < amx; index++)
	{
		for (num = 0; num < sector[index].amx; num++)
			sector[index].vertex[num].y -= len;
	}
	//lower border
	for (index = 0 ; index < amy; index++)
	{
		s = &sector[amx*(amy-1)+index];
		for (num = 0; num < s->amx; num++)
			s->vertex[s->amx*(s->amy-1)+num].y -= len;
	}
	//left border
	for (index = 0 ; index < amy; index++)
	{
		s = &sector[index*amx];
		for (num = 0; num < s->amy; num++)
			s->vertex[num*s->amx].y -= len;
	}
	//right border
	for (index = 0 ; index < amy; index++)
	{
		s = &sector[amx-1+index*amx];
		for (num = 0; num < s->amx; num++)
			s->vertex[s->amx-1+num*s->amx].y -= len;
	}
		

}

float World::getheight(float x,float y)
{
//	if (!sector) return(-999.9);
	static int indexx,indexy;	
	indexx = (int)(x/xlen);
	indexy = (int)(y/ylen);
	if (indexx >= amx || indexx < 0 || indexy >= amy || indexy < 0) return(-999.9);
	return(sector[indexx+indexy*amx].getheight(x,y));
};

void World::finalize()
{
	int xnum,ynum;


	for (xnum = 0; xnum < sectors; xnum++)
	{
		sector[xnum].finalize();
	}


	for (ynum = 0; ynum < amy; ynum++)
	for (xnum = 0; xnum < amx; xnum++)
		sector[xnum+amx*ynum].mergebordernormals((xnum+1<amx?&sector[xnum+1+ynum*amx]:NULL),((ynum+1)<amy?&sector[xnum+(ynum+1)*amx]:NULL));
}

void World::normalizetexcoords()
{
	int num;
	for (num = 0; num < sectors; num++)
	{
		sector[num].normalizetexcoords();
		sector[num].equalizetexcoords();
	}
}
void World::calculatenormals()
{
	int num;
	for (num = 0; num < sectors; num++)
	{
		sector[num].calculatenormals();
	}
}
void World::rendernormals()
{
	int num;
	for (num = 0; num < sectors; num++)
	{
		sector[num].rendernormals();
	}
}
void World::renderborders()
{
	int num;
	for (num = 0; num < sectors; num++)
	{
		sector[num].renderborder();
	}
}

void World::setviewdist(float dist,float difflen)
{
	if (difflen == -1) difflen = dist;
	save_viewdistance = dist;
	save_viewdifflen = difflen;
	viewdistance = dist + sqrt(pow(xlen/2,2)+pow(ylen/2,2));
	
	glFogf(GL_FOG_END,dist);
	glFogf(GL_FOG_START,dist-difflen);
}

void World::makerenderlist()
{

	static int renderlistlen[1024];


	static int num;
	static int pos;
	renderlistcount = 0;
	static GLVECTOR3f backward;
	static GLVECTOR3f campos;
	static int num2,num3;
	static float curlen;

	campos = *camerapos;
	backward = *camerapos;

	


	backward.x -= sin(0.0174532*cameradir->y)*cos(0.0174532*cameradir->x)*0.1;
	backward.y += sin(0.0174532*cameradir->x)*0.1;
	backward.z += cos(0.0174532*cameradir->y)*cos(0.0174532*cameradir->x)*0.1;

	


	for (num = 0; num < sectors; num++)
	{
		if (sector[num].getlen(camerapos->x,camerapos->y,camerapos->z) <= viewdistance
			&& sector[num].isinfront(&campos,&backward))
		{
			curlen = sector[num].getlen(camerapos->x,camerapos->y,camerapos->z);
			for (num2 = 0; num2 < renderlistcount; num2++)
			{
				if (curlen <= renderlistlen[num2]) break;
			}
			for (num3 = renderlistcount; num3 > num2; num3--)
			{
				renderlist[num3] = renderlist[num3-1];
				renderlistlen[num3] = renderlistlen[num3-1];
			}
			renderlist[num2] = num;
			renderlistlen[num2] = curlen;
			renderlistcount++;


//			renderlist[renderlistcount] = num;
//			renderlistcount++;

		}
		if (renderlistcount >= 1024) 
		{
			MessageBox(NULL,"To many sectors for makerenderlist to handle!!!","",MB_OK);
			exit(0);
		}

	}




}

void World::setfaceindex()
{
	int num;
	for (num = 0; num < sectors; num++)
		sector[num].setfaceindex();
}

void World::modsectorvertexes(GLVECTOR3f v)
{
	int num;
	for (num = 0; num < sectors; num++)
		sector[num].modvertex(v);
	xlen *= v.x;
	ylen *= v.z;
}





void World::render(GLenum mode)
{
	int num;
	makerenderlist();
	registerentities();
	static float sdir[4];
//	static GLVECTOR3f cdir;
	
	worldTickCount = GetTickCount();

//	cdir.set(sin(0.0174532*cameradir->y)*cos(0.0174532*cameradir->x),-sin(0.0174532*cameradir->x),-cos(0.0174532*cameradir->y)*cos(0.0174532*cameradir->x));
	for (num = 0; num < renderlistcount; num++)
	{
	

		sector[renderlist[num]].render(mode);
/*		if (!keys[SDLK_t])
			if (sector[renderlist[num]].intersectingarea(*camerapos,cdir))
			  sector[renderlist[num]].renderintersecting(*camerapos,cdir);
*/	}


	for (num = 0; num < entities; num++)
	{
		entity[num]->move(worldTickCount);
	}


		sdir[0] = 1;
		sdir[1] = 1;
		sdir[2] = 1;
		sdir[3] = 1;
		sdir[0] = sundir.x;
		sdir[1] = sundir.y;
		sdir[2] = sundir.z;
		sdir[3] = 0;

		glLightfv(GL_LIGHT0,GL_POSITION,sdir);


	for (num = 0; num < entities; num++)
	{
		if (entity[num]->homesector == ENTITY_PRERENDER) 
			entity[num]->render();
	}

	for (num = 0; num < renderlistcount; num++)
	{
		sector[renderlist[num]].renderentities();
	}

	for (num = 0; num < entities; num++)
	{
		if (entity[num]->homesector == ENTITY_STATIC) 
			entity[num]->render();
	}

	for (num = 0; num < entities; num++)
	{
		if (entity[num]->homesector == ENTITY_EFFECT) 
			entity[num]->render();
	}

	collectentities();



	


}




void World::setglobalcolor(GLVECTOR3f color)
{
	int num;
	for(num = 0; num < sectors; num++)
	{
		sector[num].fillcolor(color);
	}
}

void World::setglobalcolor2(GLVECTOR4f color)
{
	int num;
	for(num = 0; num < sectors; num++)
	{
		sector[num].fillcolor2(color);
	}
}


void World::setbump(float *map,int xsize,int ysize)
{
	int num;
	for(num = 0; num < sectors; num++)
	{
		sector[num].setbump(map,xsize,ysize);
	}
}

void World::setglobaltexture(GLuint texture)
{
	int num;
	for (num = 0; num < sectors; num++)
	{
		sector[num].filltexture(texture);
	}
}


void World::setglobaltexture2(GLuint texture)
{
	int num;
	for (num = 0; num < sectors; num++)
	{
		sector[num].filltexture2(texture);
	}
}


void World::clear()
{
	if (sector)
	{
		delete[] sector;
		sector = NULL;
	}
	if (renderlist)
	{
		delete[] renderlist;
		renderlist = NULL;
	}
}

World::World()
{
	sundir.set(3,2,6);
	sundir.normalize();
	sectors = 0;
	amx = amy = 0;
	xlen = ylen = 1;
	sector = NULL;
	renderlist = NULL;
	renderlistcount = 0;
	entities = 0;
	watercolor.set(0.6,0.6,1.0);
	fogcolor.set(1,1,1);
	waterlevel = -1;
}

World::~World()
{
	clear();
}

void World::create(int worldxsize,int worldysize,int sectorlen,float xstep,float ystep)
{
	GLVECTOR3f color(1,1,1);
	GLVECTOR4f color2(1,1,1,0);
	amx = worldxsize/sectorlen;
	amy = worldysize/sectorlen;
	xlen = ylen = sectorlen;
	sectors = amx*amy;
	sector = new Sector[sectors];
	renderlist = new int[sectors];
	int xnum,ynum;
	for (ynum = 0; ynum < amy; ynum++)
	{
		for (xnum = 0; xnum < amx; xnum++)
		{
			sector[xnum+ynum*amx].creategrid(xnum*sectorlen,ynum*sectorlen,(xnum+1)*sectorlen,(ynum+1)*sectorlen,xstep,ystep);			
			sector[xnum+ynum*amx].fillcolor(color);
			sector[xnum+ynum*amx].fillcolor2(color2);
			sector[xnum+ynum*amx].filltexture(-1);
			sector[xnum+ynum*amx].filltexture2(-1);
			sector[xnum+ynum*amx].normalizetexcoords();
			sector[xnum+ynum*amx].equalizetexcoords();
		}
	}
}


#endif